<?php

namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\NewCode;
use Teknisa\SM\Service\Sm00015004EquipeDeTrabalhoService;
use Teknisa\SM\Service\IntegracoesTeknisaService;



class Sm00015004EquipeDeTrabalhoController extends SMCrud {
    
    protected $dataSourceName = "fct_recursosagendcont_widget";
    protected $equipeService;
    protected $newCode;
    protected $integracoesTeknisaService;
    
    public function __construct($crudService, $environment, NewCode $newCode,Sm00015004EquipeDeTrabalhoService $equipeService, IntegracoesTeknisaService $integracoesTeknisaService) {
        parent::__construct($crudService, $environment);
        $this->equipeService = $equipeService;
        $this->newCode = $newCode;
        $this->integracoesTeknisaService = $integracoesTeknisaService;
    }
   
   /**
    *  Função que retona os dias da semana que estao aprorpiado para o recurso
    */
    public function find(Request\Filter $request, Response $response) {
        try {
            parent::find($request, $response);
            if(count($response->getDataSets()) <= 0){
                throw new \Exception($response->getError()->getMessage());
            } else {
                $rows = $response->getDataSets()[0]->getRows();
                foreach ($rows as &$row) {
                    $arrayDias = array();
                    if($row["IDREPETDOMINGO"] == 'S')
                        $arrayDias[] = "IDREPETDOMINGO";
                    if($row["IDREPETSEGUNDA"] == 'S')
                        $arrayDias[] = "IDREPETSEGUNDA";
                    if($row["IDREPETTERCA"] == 'S')
                        $arrayDias[] = "IDREPETTERCA";
                    if($row["IDREPETQUARTA"] == 'S')
                        $arrayDias[] = "IDREPETQUARTA";
                    if($row["IDREPETQUINTA"] == 'S')
                        $arrayDias[] = "IDREPETQUINTA";
                    if($row["IDREPETSEXTA"] == 'S')
                        $arrayDias[] = "IDREPETSEXTA";
                    if($row["IDREPETSABADO"] == 'S')
                        $arrayDias[] = "IDREPETSABADO";
                    $row["DIASDASEMANA"] = $arrayDias;
        
                };
                
                unset($row);
                $response->setDataSets(array(new DataSet($this->dataSourceName, $rows)));
            }
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     
   /**
    *  Função que salva multiplos registros de desalocações de recursos
    */
    
     public function saveDesalocacaoMultiplos(Request $request, Response $response){
        try {
            $row = $request->getRow();

            $recursos = $row["RECURSOS"];
            $dtdevagendrecurcon = $row["DTDEVAGENDRECURCON"];
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            foreach ($recursos as $recurso) {
                $nrrecursosagendcont = $recurso["NRRECURSOSAGENDCONT"];
                $this->equipeService->desalocarRecurso( $nrrecursosagendcont, $dtdevagendrecurcon, $nrorg, $cdoperador);  
            }
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
    *  Função que salva um registro de desalocaçã de um recurso
    */
    
     public function saveDesalocacao(Request $request, Response $response){
        try {
            $row = $request->getRow();
            $nrrecursosagendcont = $row["NRRECURSOSAGENDCONT"];
            $dtdevagendrecurcon = $row["DTDEVAGENDRECURCON"];
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $this->equipeService->desalocarRecurso( $nrrecursosagendcont,$dtdevagendrecurcon, $nrorg,$cdoperador);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
     
   /**
    *  Função que salva um registro de Equipe de trabalho pelo save do CRUD
    * caso o recurso da equipe de trabalho seja uma pessoa o save é feito 
    * chamando um metodo que pesquisa o calculo de intevalo de alocação
    */
    public function save(Request $request, Response $response){
        try {
            $nrrecursosagendcont = array();
            $rows       = $request->getDataSet()->getRows();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            if ($request->getDataSet()->getRows()[0]["__is_new"]) {
                $msgError = $this->equipeService->validarDiasAlocacaoItens($rows, $nrorg);
                if($msgError) {
                    throw new \Exception("Operação bloqueada!<br>" . $msgError);
                } else {
                    foreach ($rows as $key=>&$row) {
                        $row["NRRECURSOSAGENDCONT"] = $this->newCode->getCodeByTableName("FCT_RECURSOSAGENDCONT", $nrorg, 1, 10, true);
                        array_push($nrrecursosagendcont, $row["NRRECURSOSAGENDCONT"]);
                        $this->equipeService->inserirRecursos($row);
                    }
                    unset($row);
                }
            } else {
                parent::save($request, $response);
                foreach ($rows as $row) {
                    if($row["NRPARCNEPESSOA"] !== null){
                        $nrparcinterlocutor  = null;
                        $cdoperador          = $this->environment->getCdOperador();
                        $inserindo           = $row["__is_new"];
                        $nrrecursosagendcont = $row["NRRECURSOSAGENDCONT"];
                        $dataInicial         = $row["DTINIAGENDRECURSO"];
                        $dataFinal           = $row["DTFIMAGENDRECURSO"];
                        $nomeDiasSemana      = $row["DIASDASEMANA"];
                        $nrparcnegrecurso    = $row["NRPARCNEPESSOA"];
                        $nrseqcontrato       = isset($row["NRSEQCONTRATO"]) ? $row["NRSEQCONTRATO"] : $this->equipeService->getNrContrato($nrrecursosagendcont, $nrorg);
                        $descansoRemunerado  = $row["IDDESCANSOREMU"];
                        
                        if($row["IDDESCANSOREMU"] == "S")
                            $this->equipeService->calculoIntervalo($inserindo, $nomeDiasSemana, $dataInicial, $dataFinal, $nrrecursosagendcont, $nrparcnegrecurso, $nrparcinterlocutor, $nrorg, $cdoperador, $nrseqcontrato, $descansoRemunerado);
                        if($row["IDESCALATRAB"] == "M")
                            $this->equipeService->salvarApropiacaoEdicaoEscala($row, $nrorg, $cdoperador, $row["DTINIAGENDRECURSO"], $row["DTFIMAGENDRECURSO"]);
                    }
                }
            }
            
            if(!is_array($nrrecursosagendcont))
                $nrrecursosagendcont = array($nrrecursosagendcont);
                
            $response->addDataSet(new DataSet("NRRECURSOSAGENDCONT", $nrrecursosagendcont));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    /**
     * Gera o Array de parâmetros e retonar para requisição no WebServiceHCM
     * @params{String, String}
    **/
    public function getArray($dtfererencia){
         $webServiceHcm = array(
                    "DTMESCOMPETENC"  => $dtfererencia,
                    "CDCNPJESTRUT"    => "",
                    "CDFILIAL"        => "",
                    "CDCENTROCUSTO"   => "",
                    "NRCPFPESSOA"     => "",
                    "DTREFERENCIA"    => ""
                
        );
        return $webServiceHcm;
    }
    
    public function getPessoasHCM($nrorg, $dtreferencia){
        $rota          = "retornaDadosPessoa";
        $urlintegracao = $this->equipeService->getUrlIntegracaoHcm($nrorg);
        $rowDados = $this->getArray($dtreferencia);
        $retorno = $this->integracoesTeknisaService->enviarDadosIntegracaoHCM($urlintegracao, $rota, $rowDados);
        return $retorno;
    } 
        
    public function sincronizarRecurso(Request\Filter $request,Response $response){
        try {
            $this->dataSourceName = 'parcnegocio_get';

            $filtro = $request->getFilterCriteria()->getConditions();
            
            $nrorg = $filtro[0]["value"];
            $dtreferencia = $filtro[2]["value"];
            
            parent::find($request, $response);
            
            $parceiroSM =$response->getDataSets()[0]->getRows();
            $pessoasHCM = $this->getPessoasHCM($nrorg, $dtreferencia);
            $pessoasInex = [];

            foreach ($pessoasHCM["pessoas"] as $key => $pessoa) {
                $existe = true; 
                foreach($parceiroSM as $parc){
                    if($parc["NRINSCRICAOPARC"] == $key){
                        $existe = false;
                        break;
                    }
                }
                if($existe)
                    array_push($pessoasInex, $pessoa);
            };

            foreach($pessoasInex as $pessoa) {
                $this->equipeService->saveParceiro($pessoa);
            }
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    /**
     * Salva a escala de trabalho inserida manualmente pelo usuário
    **/
    public function saveEscalaTrabalho(Request $request, Response $response){
        try{
            $rows       = $request->getDataSet()->getRows();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            $this->equipeService->saveEscalaTrabalho($rows, $nrorg, $cdoperador);
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
        /**
     * Salva a edição de uma escala de trabalho inserida manualmente pelo usuário
    **/
    public function saveEdicaoEscalaTrabalho(Request $request, Response $response){
        try {
            $rows       = $request->getDataSet()->getRows();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();

            $this->equipeService->saveEdicaoEscalaTrabalho($rows, $nrorg, $cdoperador);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * deleta uma escala de trabalho inserida manualmente pelo usuário
    **/
    public function deletarEscala(Request\Row $request, Response $response){
        try {
            $row                 = $request->getRow();
            $nrrecursosagendcont = $row["NRRECURSOSAGENDCONT"];
            $dtentrada           = $row["DTENTRADA"];
            $nrorg               = $this->environment->getNrOrg();

            $this->equipeService->deletarEscala($nrrecursosagendcont, $nrorg, $dtentrada);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
     
    /**
     * Valida o perído de alocação para verificar se o recurso escolhido
     * esta disponível naquela
    **/  
    public function verificaDisponibilidadeRec(Request $request, Response $response){
        try {
            $rows       = $request->getDataSet()->getRows();
            $nrorg      = $this->environment->getNrOrg();

            $disponibilidade = $this->equipeService->verificaDisponibilidadeRec($rows, $nrorg);
            $response->addDataSet(new DataSet("fct_recursosagendcont_widget", array("DISPONIBILIDADE" => $disponibilidade)));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}

?>

