<?php

namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\SM\Util\Procedure\FctFaturaContrato;
use Teknisa\SM\Util\Procedure\FctCancFaturaContrato;
use Teknisa\SM\Util\Procedure\FctCancFaturaContratoApu;
//use Teknisa\SM\Util\Procedure\FctApurarFaturamento;
use Teknisa\SM\Util\Procedure\FctApurarFaturamentoLista;
use Teknisa\SM\Util\Procedure\FctCancApuracaoFaturamento;
use Teknisa\SM\Service\Sm00001000LancamentoPropostaService;
use Teknisa\SM\Factory\Sm00010000FaturamentoApuracaoFactory;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\SM\Service\IntegracoesTeknisaService;
use Teknisa\SM\Util\Utilities;
use Teknisa\Libs\Util\NewCode;


class Sm00010000FaturamentoApuracaoService {
    
    protected $connection;
    protected $entityManager;
    protected $propostaService;
    protected $apuracaoFaturamentoFactory;
    protected $integracoesTeknisaService;
    protected $validaReajusteFatu;
    
    public function __construct(EntityManager $entityManager, Sm00001000LancamentoPropostaService $propostaService, Sm00010000FaturamentoApuracaoFactory $apuracaoFaturamentoFactory, IntegracoesTeknisaService $integracoesTeknisaService, NewCode $newCode, $validaReajusteFatu) {
        
        $this->connection                 = $entityManager->getConnection();
        $this->entityManager              = $entityManager;
        $this->propostaService            = $propostaService;
        $this->apuracaoFaturamentoFactory = $apuracaoFaturamentoFactory;
        $this->integracoesTeknisaService  = $integracoesTeknisaService;
        $this->newCode                    = $newCode;
        $this->validaReajusteFatu         = $validaReajusteFatu;

    }

    public function invoiceValidateHasAlreadyBeenUsed($nrLanctoNF,$cdFilial){
        /** @var \Teknisa\Model\Services\Repositories\FctApuracao $repositoryFctApuracao */
        $repositoryFctApuracao = $this->entityManager->getRepository(Utilities::SERVICES_ENTITY_DIR . 'FctApuracao');
        return $repositoryFctApuracao->invoiceValidateHasAlreadyBeenUsed($nrLanctoNF,$cdFilial);
    }
    
    /**
     * Executa a procedure "FCT_FATURA_CONTRATO".
     */
    public function executarFctFaturaContrato($nrseqcontrato, $nrparcnegocio, $dtinicial, $dtfinal, $nrvencprevcont, $nrgrupofatura, $nrrateioitemvenc, $nrorg, $operador) {
        $procedure = new FctFaturaContrato($this->connection);
        $procedure->executar($nrseqcontrato, $nrparcnegocio, $dtinicial, $dtfinal, $nrvencprevcont, $nrgrupofatura, $nrrateioitemvenc, $nrorg, $operador);
    }
    
    /**
     * Atualiza o valor do vencimento de acordo com a composição de serviço 
     * da proposta e os tipos de medida variáveis.
     */
    public function atualizarValoresVencimento($nrvencprevcont, $nrseqcontrato, $nrorg, $medidas) {
        $proposta = $this->getPropostaDoContrato($nrseqcontrato, $nrorg);
        
        $nrseqorcamento    = $proposta["NRSEQORCAMENTO"];
        $nrorcamentomodelo = $proposta["NRORCAMENTOMODELO"];
        $nrtipoorcamento   = $proposta["NRTIPOORCAMENTO"];
        
        $escopos = array($nrtipoorcamento);
        
        foreach ($medidas as &$medida) {
            $medida["CDTPMEDIDASERV"] = $medida["CDTPMEDIDAVARIAVEL"];
            $medida["QUANTIDADE"]     = $medida["QTMEDIDAVARIAVEL"];
            
            unset($medida["CDTPMEDIDAVARIAVEL"]);
            unset($medida["QTMEDIDAVARIAVEL"]);
        }
        unset($medida);
        
        $variacoesPreco = $this->propostaService->getVariacoesPreco($escopos, $nrorg);
        $variacoesPreco = $this->propostaService->filtrarVariacoesPreco($medidas, $variacoesPreco);
        
        $elementosEscolhidos = $this->propostaService->getElementosEscolhidos($nrseqorcamento, $nrorg);
        $composicoes = $this->propostaService->getComposicoesProposta($nrseqorcamento, $nrorg);
        
        $elementosSemCodComposicao = $this->propostaService->getElementosComposicaoSemCodigoComposicao($escopos, $composicoes);
        $elementosSemCodComposicao = $this->propostaService->eliminarElementosDuplicados($elementosSemCodComposicao);
        $elementosSemCodComposicao = $this->propostaService->preencherIdEscolhido($elementosSemCodComposicao, $elementosEscolhidos);
        
        if (count($variacoesPreco) > 0) {
            $elementosSemCodComposicao = $this->propostaService->atualizarValoresElementosComposicao($variacoesPreco, $elementosSemCodComposicao);
        }
        
        $servicos = $this->propostaService->getServicos($elementosSemCodComposicao);
        
        $elementosAlterados = $this->propostaService->getElementosAlteradosPeloUsuarioNaComposicaoServico($nrseqorcamento, $nrorg);
        
        if (count($elementosAlterados) > 0) {
            foreach ($elementosAlterados as $elementoAlterado) {
                foreach ($servicos as &$serv) {
                    if ($serv["CDSERVORCA"] == $elementoAlterado["CDSERVORCA"] && $serv["NRTIPOORCAMENTO"] == $elementoAlterado["NRTIPOORCAMENTO"]) {
                        $serv["QTMEDIDASERV"] = $serv["QTMEDIDASERV"] + $elementoAlterado["QTMEDSERVORCA"];
                        $serv["VRTOTALITEMORCA"] = $serv["VRUNITITEMORCA"] * $serv["QTMEDIDASERV"];
                    }
                }
            }
        }
        unset($serv);
        
        $itens = $this->getItensContrato($nrseqcontrato, $nrorg);
        
        foreach ($servicos as &$serv) {
            foreach ($itens as $item) {
                if ($serv["CDSERVORCA"] == $item["CDSERVORCA"] && $serv["NRTIPOORCAMENTO"] == $item["NRTIPOORCAMENTO"]) {
                    $serv["QTITEMORCAMENTO"] = $item["QTITEMCONTRATO"];
                    $serv["IDTPDESCORCA"]    = $item["IDTPDESCONTO"];
                    $serv["VRPERCDESCORCA"]  = $item["VRDESCONTO"];
                    $serv["QTMEDIDASERV"]    = $item["QTMEDIDACONTR"];
                    
                    $serv["VRITEMSEMDESC"] = $serv["VRUNITITEMORCA"] * $serv["QTITEMORCAMENTO"] * $serv["QTMEDIDASERV"];
                    
                    switch ($serv["IDTPDESCORCA"]) {
                        case "S": {
                            $serv["VRTOTALITEMORCA"] = $serv["VRITEMSEMDESC"];
                            $serv["VRPERCDESCORCA"] = 0;
                            break;
                        }
                        case "P": {
                            $serv["VRTOTALITEMORCA"] = $serv["VRITEMSEMDESC"] - ($serv["VRITEMSEMDESC"] * $serv["VRPERCDESCORCA"] / 100);
                            break;
                        }
                        case "V": {
                            if ($serv["VRITEMSEMDESC"] >= $serv["VRPERCDESCORCA"]) {
                                $serv["VRTOTALITEMORCA"] = $serv["VRITEMSEMDESC"] - $serv["VRPERCDESCORCA"];
                            }
                            else {
                                $serv["VRTOTALITEMORCA"] = $serv["VRITEMSEMDESC"];
                                $serv["IDTPDESCORCA"] = "N";
                                $serv["VRPERCDESCORCA"] = 0;
                            }
                            break;
                        }
                        case "Y": {
                            $serv["VRTOTALITEMORCA"] = $serv["VRITEMSEMDESC"] + ($serv["VRTOTALITEMORCA"] * $serv["VRPERCDESCORCA"] / 100);
                            break;
                        }
                        case "X": {
                            $serv["VRTOTALITEMORCA"] = $serv["VRITEMSEMDESC"] + $serv["VRPERCDESCORCA"];
                            break;
                        }
                    }
                }
            }
        }
        unset($serv);
        
        $itensVencimento = $this->getItensFaturamento($nrvencprevcont, $nrseqcontrato, $nrorg);
        
        $itensVencimentoComValor = $this->mergeServicosComItensVencimento($servicos, $itensVencimento);
        
        $paramsVenc = $this->getParametrosVencimento($nrvencprevcont, $nrseqcontrato, $nrorg);
        
        $idtpfatuincomp = $paramsVenc["IDTPFATUINCOMP"];
        
        $dtiniciocontrato = $paramsVenc["DTINICIOCONTRATO"];
        $dtfimcontrato    = $paramsVenc["DTFIMCONTRATO"];
        $dtinicompet      = $paramsVenc["DTINICOMPET"];
        $dtfincompet      = $paramsVenc["DTFINCOMPET"];
        
        $mesAnoInicialContrato = date_create_from_format("d/m/Y", $dtiniciocontrato)->format("m/Y");
        $mesAnoInicialCompetencia = date_create_from_format("d/m/Y", $dtinicompet)->format("m/Y");
        
        $mesAnoFinalContrato = date_create_from_format("d/m/Y", $dtfimcontrato)->format("m/Y");
        $mesAnoFinalCompetencia = date_create_from_format("d/m/Y", $dtfincompet)->format("m/Y");
        
        //verifica se a competência atual é a primeira ou a última, para recalcular o valor dos itens do vencimento proporcional a quantidade de dias do mês
        if ($mesAnoInicialContrato == $mesAnoInicialCompetencia || $mesAnoFinalContrato == $mesAnoFinalCompetencia) {
            $itensVencimentoComValor = $this->recalcularValorItensVencimento($idtpfatuincomp, $itensVencimentoComValor, $dtinicompet, $dtfincompet);
        }
        
        $this->atualizarValorItensVencimento($itensVencimentoComValor);
        $valorTotalItens = $this->getValorTotalItensVencimento($nrvencprevcont, $nrseqcontrato, $nrorg);
        $this->atualizarValorTotalVencimento($valorTotalItens, $nrvencprevcont, $nrseqcontrato, $nrorg);
    }
    
    /**
     * Retorna os tipos de medida variáveis da proposta que gerou o contrato.
     */
    public function getMedidasVariaveisDaPropostaDoContrato($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        return $this->apuracaoFaturamentoFactory->getMedidasVariaveisDaPropostaDoContrato($params);
    }
    
    /**
     * Retorna a proposta que gerou o contrato.
     */
    public function getPropostaDoContrato($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->apuracaoFaturamentoFactory->getPropostaDoContrato($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    /**
     * Retorna os itens do vencimento
     */
    public function getItensFaturamento($nrvencprevcont, $nrseqcontrato, $nrorg) {
        $params = array(
            "NRVENCPREVCONT" => $nrvencprevcont,
            "NRSEQCONTRATO"  => $nrseqcontrato,
            "NRORG"          => $nrorg
        );
        
        return $this->apuracaoFaturamentoFactory->getItensFaturamento($params);
    }
    
    /**
     * Seta os valores dos serviços nos itens do vencimento
     */
    public function mergeServicosComItensVencimento($servicos, $itensVencimento) {
        for ($i = 0; $i < count($itensVencimento); $i++) {
            for ($j = 0; $j < count($servicos); $j++) {
                if ($itensVencimento[$i]["CDSERVORCA"] == $servicos[$j]["CDSERVORCA"]) {
                    $itensVencimento[$i]["VRVENCIMENTO"] = $servicos[$j]["VRTOTALITEMORCA"];
                    break;
                }
            }
        }
        
        return $itensVencimento;
    }
    
    /**
     * Atualiza no banco o valor dos itens do vencimento
     */
    public function atualizarValorItensVencimento($itensVencimento) {
        foreach ($itensVencimento as $item) {
            $params = array(
                "NRVENCPREVCONT"   => $item["NRVENCPREVCONT"],
                "NRITVENCPREVCONT" => $item["NRITVENCPREVCONT"],
                "NRSEQCONTRATO"    => $item["NRSEQCONTRATO"],
                "VRVENCIMENTO"     => $item["VRVENCIMENTO"],
                "NRORG"            => $item["NRORG"]
            );
            
            $this->apuracaoFaturamentoFactory->atualizarItemVencimento($params);
        }
    }
    
    /**
     * Atualiza no banco o valor total do vencimento
     */
    public function atualizarValorTotalVencimento($valorTotalItens, $nrvencprevcont, $nrseqcontrato, $nrorg) {
        $params = array(
            "VALOR"          => $valorTotalItens,
            "NRVENCPREVCONT" => $nrvencprevcont,
            "NRSEQCONTRATO"  => $nrseqcontrato,
            "NRORG"          => $nrorg
        );
        
        $this->apuracaoFaturamentoFactory->atualizarValorTotalVencimento($params);
    }
    
    /**
     * Atualiza no banco o valor total do vencimento
     */
    public function atualizarValorItVencHora($valorTotalItens, $nrvencprevcont, $nrseqcontrato, $nritemocontrato, $nrorg) {
        $params = array(
            "VRVENCIMENTO"     => $valorTotalItens,
            "NRVENCPREVCONT"   => $nrvencprevcont,
            "NRSEQCONTRATO"    => $nrseqcontrato,
            "NRITEMOCONTRATO"  => $nritemocontrato,
            "NRORG"            => $nrorg
        );
        
        $this->apuracaoFaturamentoFactory->atualizarItemVencimentoHoras($params);
    }
    
    
       /**
     * limpa o valor do item do vencimento
     */
    public function limparVrItemVencimentoHoras($nrvencprevcont, $nrseqcontrato, $nritemocontrato, $nrorg) {
        $params = array(
            "NRVENCPREVCONT"   => $nrvencprevcont,
            "NRSEQCONTRATO"    => $nrseqcontrato,
            "NRITEMOCONTRATO"  => $nritemocontrato,
            "NRORG"            => $nrorg
        );
        
        $this->apuracaoFaturamentoFactory->limparVrItemVencimentoHoras($params);
    }
    
    
    /**
     * Executa a procedure "FCT_CANC_FATURA_CONTRATO".
     */
    public function executarProcedureFctCancFaturaContrato($nrseqcontrato, $dataini, $datafin, $nrvencprevcont, $nrrateioitemvenc, $nrorg, $cdoperador) {
        $procedure = new FctCancFaturaContrato($this->connection);
        $procedure->executar($nrorg, $nrseqcontrato, $cdoperador, $dataini, $datafin, $nrvencprevcont, $nrrateioitemvenc);
    }
    
    /**
     * Executa a procedure "FCT_CANC_FATURA_CONTRATO_APU".
     */
    public function executarProcedureFctCancFaturaContratoApu($nrseqcontrato, $nrvencprevcont, $nrapuracao, $nrorg) {
        $procedure = new FctCancFaturaContratoApu($this->connection);
        $procedure->executar($nrorg, $nrseqcontrato, $nrvencprevcont, $nrapuracao);
    }
    
    /**
     * Retorna os itens do vencimento com as datas de início e fim do contrato e da competência atual
     */
    public function getParametrosVencimento($nrvencprevcont, $nrseqcontrato, $nrorg) {
        $params = array(
            "NRVENCPREVCONT" => $nrvencprevcont,
            "NRSEQCONTRATO"  => $nrseqcontrato,
            "NRORG"          => $nrorg
        );
        
        $dataset = $this->apuracaoFaturamentoFactory->getParametrosVencimento($params);
        return $dataset[0];
    }
    
    /**
     * Recalcula o valor dos itens do vencimento para os períodos incompletos
     */
    private function recalcularValorItensVencimento($idtpfatuincomp, $itensVencimentoComValor, $dtinicompet, $dtfincompet) {
        $dtinicompet = DateUtil::getDataDeString($dtinicompet, DateUtil::FORMATO_BRASILEIRO, true);
        $dtfincompet = DateUtil::getDataDeString($dtfincompet, DateUtil::FORMATO_BRASILEIRO, true);
        
        $periodo = (int)$dtinicompet->format("t") - 1;
        $diferenca = DateUtil::getQtdeDiasEntre($dtinicompet, $dtfincompet);
        if ($idtpfatuincomp != "2") {
            
            if ($diferenca < $periodo) {
                foreach ($itensVencimentoComValor as &$item) {
                    $valorVencimento = $item["VRVENCIMENTO"];
                    
                    $valorVencimento = $valorVencimento / ($periodo + 1) * ($diferenca + 1);
                    $item["VRVENCIMENTO"] = $valorVencimento;
                }
                unset($item);
            }
        }
        
        return $itensVencimentoComValor;
    }
     /**
     * Recalcula o valor dos itens do vencimento para os períodos incompletos
     */
    //public function gerarFaturamentoHora($pessoa, $cdoperadorlogado, $nrorg) {
    public function gerarFaturamentoHora($nrseqcontrato, $nritemocontrato, $pessoaCPF, $nrparcnepessoa, $mescompet, $nrvencprevcont, $nrorg) {
        $dtfererencia = "01/".$mescompet;
        $webServiceHcm = $this->getArray($dtfererencia, $pessoaCPF);
        $retorno       = $this->enviarElementosHcm($nrorg, $webServiceHcm);
        
        if(!$retorno){
            return ("Falha na integração com o HCM.");
        }else{
            foreach ($retorno["itens"] as $value) {
                $this->saveTabelaTemporaria($nrseqcontrato, $nritemocontrato, $nrparcnepessoa, $dtfererencia, $value["IDTIPOEVENTO"], $value["IDUNIDREFEREVEN"], $value["VRREFFOLHA"], $value["NREVENTOM"], $nrvencprevcont, $nrorg);
           }
        return true;
        }
    }
    
      /**
     * Envia os dados de evolução de oportunidade para o CRM
     * @params{Array} rowDados dados que serão enviados
    **/
    public function enviarElementosHcm($nrorg, $rowDados) {
        $urlintegracao = $this->getUrlIntegracaoHcm($nrorg);
        $rota          = "retornaLancamentosDeCalculo";
        
        $retorno = $this->integracoesTeknisaService->enviarDadosIntegracaoHCM($urlintegracao, $rota, $rowDados);

        if (isset($retorno["itens"])) {
            return $retorno;
        } else {
            return false;
        }
    }
    
    /**
     * Busca por dados de acordo com os enviados
     * @params{interger, string}
    **/
    public function getEventos($nrvencprevcont, $nrseqcontrato, $pessoa, $nrorg, $eventos) {
        $params = array(
            "NRVENCPREVCONT"      => $nrvencprevcont,
            "NRSEQCONTRATO"       => $nrseqcontrato,
            "NRPARCNEGOCIOPESSOA" => $pessoa,
            "NRORG"               => $nrorg,
            "NRSEVENTOS"          => $eventos
        );
        
        $dados = $this->apuracaoFaturamentoFactory->getEventos($params);
        return $dados;
    }
    
    
    /**
     * Busca por dados de acordo com os enviados
     * @params{interger, string}
    **/
    public function buscaAgendamentosDeRecurso($nrseqcontrato, $nrorg, $dtinicompet, $dtfincompet) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg,
            "DATAINI"       => $dtinicompet,
            "DATAFIN"       => $dtfincompet
        );
        
        return $this->apuracaoFaturamentoFactory->buscaAgendamentosDeRecurso($params);
    }
    /**
     * Prepara os dados para realizar o save
     * @params{polyTipos}
    **/
    public function saveTabelaTemporaria($nrseqcontrato, $nritemocontrato, $nrparcnepessoa, $dtreferencia, $idtipoevento, $idunidrefereven, $vrreffolha, $nreventom, $nrvencprevcont, $nrorg) {
        $completaFormato = " 00:00:00";
        $params = array(
            "NRSEQCONTRATO"       => $nrseqcontrato,
            "NRITEMOCONTRATO"     => $nritemocontrato,
            "NRPARCNEGOCIOPESSOA" => $nrparcnepessoa,
            "NRVENCPREVCONT"      => $nrvencprevcont,
            "DTREFERENCIA"        => $dtreferencia.$completaFormato,
            "IDTIPOEVENTO"        => $idtipoevento,
            "IDUNIDREFEREVEN"     => $idunidrefereven,
            "VRREFFOLHA"          => $vrreffolha,
            "NREVENTOM"           => $nreventom,
            "NRORG"               => $nrorg
        );
       $this->apuracaoFaturamentoFactory->saveTabelaTemporaria($params);
    }
    
    
    /**
     * Deleta os dados na tabela temporaria
    **/
    public function deleteTabelaTemporaria($nrvencprevcont, $nrseqcontrato, $nrorg) {
        $params = array(
            "NRVENCPREVCONT"      => $nrvencprevcont,
            "NRSEQCONTRATO"       => $nrseqcontrato,
            "NRORG"               => $nrorg
        );
       $this->apuracaoFaturamentoFactory->deleteTabelaTemporaria($params);
    }
    
     /**
     * Valida os dados na tabela temporariae
    **/
    public function validaTabelaTemporaria($nrvencprevcont, $nrseqcontrato, $pessoa, $nrorg) {
        $params = array(
            "NRSEQCONTRATO"       => $nrseqcontrato,
            "NRVENCPREVCONT"      => $nrvencprevcont,
            "NRPARCNEGOCIOPESSOA" => $pessoa,
            "NRORG"               => $nrorg
        );
       $dados = $this->apuracaoFaturamentoFactory->validaTabelaTemporaria($params);
       return $dados[0]["VALIDA"];
    }
    
    
     /**
     * Gera o Array de parâmetros e retonar para requisição no WebServiceHCM
     * @params{String, String}
    **/
    public function getArray($dtfererencia, $pessoaCPF){
         $webServiceHcm = array(
                    "DTMESCOMPETENC"  => $dtfererencia,
                    "NRTIPOMOVIMENT"  => "",
                    "NROCORRECAL"     => "",
                    "CDCNPJESTRUT"    => "",
                    "CDFILIAL"        => "",
                    "CDCENTROCUSTO"   => "",
                    "NRCPFPESSOA"     => $pessoaCPF,
                    "NREVENTOM"       => "10,20,70,74,90,810,970,974,990,1040,1041,1050",
                    "DTOCORRENCIAITEM"=> ""
                
        );
        return $webServiceHcm;
    }
     /**
     * Busca o CPF das pessoas do Contrato retorna um dataset com todos CPF
     * @params{Interger, DateTime, Interger}
    **/
    public function getCPFpessoas($nrvencprevcont, $dtinicompet, $dtfincompet, $nrorg) {
        $params = array(
            "NRVENCPREVCONT" => $nrvencprevcont,
            "DTINICOMPET"    => $dtinicompet,
            "DTFINCOMPET"    => $dtfincompet,
            "NRORG"          => $nrorg
        );
        $dataset = $this->apuracaoFaturamentoFactory->getCPFpessoas($params);
        
        return $dataset;
    }
    
     /**
     * Busca o codigo do intem do Contrato retorna um valor do intem
     * @params{Interger, Interger, Interger}
    **/
    
    public function getItenDoContrato($nrseqcontrato, $nrvencprevcont, $nrorg) {
        $params = array(
            "NRSEQCONTRATO"  => $nrseqcontrato,
            "NRVENCPREVCONT" => $nrvencprevcont,
            "NRORG"          => $nrorg
        );
        $itens = $this->apuracaoFaturamentoFactory->getItenDoContrato($params);
        
        return $itens;
    }
     /**
     * Retorna a url de integração com o crm
     * @params $nrorg nr do modelo
    **/
    public function getUrlIntegracaoHcm($nrorg){
        $param = array(
            "NRORG" => $nrorg
        );
        
        $dataset = $this->apuracaoFaturamentoFactory->getUrlIntegracaoHcm($param);
        return $dataset[0]["URLINTEGRACAOHCM"];
    }
    /**
     * Retorna o total de horas agendadas para o contrato.
     */
    public function getTotalHorasAgendadasContrato($nrseqcontrato, $dataini, $datafin, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg,
            "DATAINI"       => $dataini,
            "DATAFIN"       => $datafin
        );
        
        $dataset = $this->apuracaoFaturamentoFactory->getTotalHorasAgendadasContrato($params);
        return $dataset[0]["HORAS"];
    }
    
    /**
     * Retorna o total de horas agendadas para o contrato.
     */
    /*public function getTotalHorasAgendadasContratoHora($nrseqcontrato, $nritemocontrato, $nrorg, $dtapropriacao, $pessoa) {
        $params = array(
            "NRSEQCONTRATO"   => $nrseqcontrato,
            "NRITEMOCONTRATO" => $nritemocontrato,
            "NRORG"           => $nrorg,
            "DTAPROPRIACAO"   => $dtapropriacao,
            "NRPARCNEPESSOA"  => $pessoa
        );
        
        $dataset = $this->apuracaoFaturamentoFactory->getTotalHorasAgendadasContratoHora($params);
        return $dataset[0]["TEMPOBASE"];
    }*/
    
    /**
     * Retorna o total de horas apontadas nos agendamentos do contrato.
     */
    public function getTotalHorasApontadasContrato($nrseqcontrato, $dataini, $datafin, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg,
            "DATAINI"       => $dataini,
            "DATAFIN"       => $datafin
        );
        
        $dataset = $this->apuracaoFaturamentoFactory->getTotalHorasApontadasContrato($params);
        return $dataset[0]["HORAS"];
    }
    
    /**
     * Verifica se o contrato usa as horas apontadas nos agendamentos no faturamento
     */
    public function verificarApuraApontamentoHoras($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->apuracaoFaturamentoFactory->getIdApuraApontamentoContrato($params);
        return ($dataset[0]["IDAPURAAPONTAMENTO"] == "S");
    }
    
    public function getValorVencimento($nrvencprevcont, $nrseqcontrato, $nrorg) {
        $params = array(
            "NRVENCPREVCONT" => $nrvencprevcont,
            "NRSEQCONTRATO"  => $nrseqcontrato,
            "NRORG"          => $nrorg
        );
        
        $dataset = $this->apuracaoFaturamentoFactory->getValorVencimento($params);
        return $dataset[0]["VRVENCIMENTO"];
    }
    
    public function getValorTotalItensVencimento($nrvencprevcont, $nrseqcontrato, $nrorg) {
        $params = array(
            "NRVENCPREVCONT" => $nrvencprevcont,
            "NRSEQCONTRATO"  => $nrseqcontrato,
            "NRORG"          => $nrorg
        );
        
        $dataset = $this->apuracaoFaturamentoFactory->getValorTotalItensVencimento($params);
        return $dataset[0]["VRVENCIMENTO"];
    }
    
    public function getItensContrato($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        return $this->apuracaoFaturamentoFactory->getItensContrato($params);
    }
    
    public function getItensContratoHoras($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        return $this->apuracaoFaturamentoFactory->getItensContratoHoras($params);
    }
    
    public function atualizarValoresVencimentoHoras($nrvencprevcont, $nrseqcontrato, $nrorg) {
        
        $itens = $this->getItensContratoHoras($nrseqcontrato, $nrorg);
        
        $itensVencimento = $this->getItensFaturamento($nrvencprevcont, $nrseqcontrato, $nrorg);
        
        $itensVencimentoComValor = $this->mergeItensContComItensVencimento($itens, $itensVencimento);
        
        $paramsVenc = $this->getParametrosVencimento($nrvencprevcont, $nrseqcontrato, $nrorg);
        
        $idtpfatuincomp = $paramsVenc["IDTPFATUINCOMP"];
        
        $dtiniciocontrato = $paramsVenc["DTINICIOCONTRATO"];
        $dtfimcontrato    = $paramsVenc["DTFIMCONTRATO"];
        $dtinicompet      = $paramsVenc["DTINICOMPET"];
        $dtfincompet      = $paramsVenc["DTFINCOMPET"];
        
        $mesAnoInicialContrato = date_create_from_format("d/m/Y", $dtiniciocontrato)->format("m/Y");
        $mesAnoInicialCompetencia = date_create_from_format("d/m/Y", $dtinicompet)->format("m/Y");
        
        $mesAnoFinalContrato = date_create_from_format("d/m/Y", $dtfimcontrato)->format("m/Y");
        $mesAnoFinalCompetencia = date_create_from_format("d/m/Y", $dtfincompet)->format("m/Y");
        
        //verifica se a competência atual é a primeira ou a última, para recalcular o valor dos itens do vencimento proporcional a quantidade de dias do mês
        if ($mesAnoInicialContrato == $mesAnoInicialCompetencia || $mesAnoFinalContrato == $mesAnoFinalCompetencia) {
            $itensVencimentoComValor = $this->recalcularValorItensVencimento($idtpfatuincomp, $itensVencimentoComValor, $dtinicompet, $dtfincompet);
        }
        
        $this->atualizarValorItensVencimento($itensVencimentoComValor);
        $valorTotalItens = $this->getValorTotalItensVencimento($nrvencprevcont, $nrseqcontrato, $nrorg);
        $this->atualizarValorTotalVencimento($valorTotalItens, $nrvencprevcont, $nrseqcontrato, $nrorg);
    }
    
    /**
     * Seta os valores dos itens do contrato nos itens do vencimento
     */
    public function mergeItensContComItensVencimento($itens, $itensVencimento) {
        for ($i = 0; $i < count($itensVencimento); $i++) {
            for ($j = 0; $j < count($itens); $j++) {
                if ($itensVencimento[$i]["CDSERVORCA"] == $itens[$j]["CDSERVORCA"]) {
                    $itensVencimento[$i]["VRVENCIMENTO"] = $itens[$j]["VRTOTITEMCONT"];
                    break;
                }
            }
        }
        
        return $itensVencimento;
    }
    
    public function salvarTabelaRelatorioMedidaFaturamento($nrseqcontrato, $nritemocontrato, $nrvencprevcont, $nrparcnegpessoa, $nrhorasagendadas, $nrhorasdia, $nrhorastrabalhadas, $vrtrabalhado, $qthrextra50, $vrhrextra50, $qthrextra70, $vrhrextra70, $qthrextra100, $vrhrextra100, $qttothrextras, $vrtothrextras, $vrtotmedicao, $nrorg) {
        $params = array(
            "NRSEQCONTRATO"      => $nrseqcontrato,
            "NRITEMOCONTRATO"    => $nritemocontrato,
            "NRVENCPREVCONT"     => $nrvencprevcont,
            "NRPARCNEGPESSOA"    => $nrparcnegpessoa,
            "NRHORASAGENDADAS"   => $nrhorasagendadas,
            "NRHORASDIA"         => $nrhorasdia,
            "NRHORASTRABALHADAS" => $nrhorastrabalhadas,
            "VRTRABALHADO"       => $vrtrabalhado,
            "QTHREXTRA50"        => $qthrextra50,
            "VRHREXTRA50"        => $vrhrextra50,
            "QTHREXTRA70"        => $qthrextra70,
            "VRHREXTRA70"        => $vrhrextra70,
            "QTHREXTRA100"       => $qthrextra100,
            "VRHREXTRA100"       => $vrhrextra100,
            "QTTOTHREXTRAS"      => $qttothrextras,
            "VRTOTHREXTRAS"      => $vrtothrextras,
            "VRTOTMEDICAO"       => $vrtotmedicao,
            "NRORG"              => $nrorg
        );
        
        $this->apuracaoFaturamentoFactory->salvarTabelaRelatorioMedidaFaturamento($params);
    }
    
    public function getDadosItemVencimento($nrseqcontrato, $nritemocontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO"   => $nrseqcontrato,
            "NRITEMOCONTRATO" => $nritemocontrato,
            "NRORG"           => $nrorg
        );
        
        $dataset = $this->apuracaoFaturamentoFactory->getDadosItemVencimento($params);
        return $dataset[0];
    }
    
    public function calculaHoras($nrvencprevcont, $nrseqcontrato, $nrorg, $dtinicompet, $dtfincompet) {
        $agendamentos = $this->buscaAgendamentosDeRecurso($nrseqcontrato, $nrorg, $dtinicompet, $dtfincompet);
        if (count($agendamentos) > 0) {
            
            $this->atualizarValoresVencimentoHoras($nrvencprevcont, $nrseqcontrato, $nrorg);
            //$valorVencimento = $this->getValorVencimento($nrvencprevcont, $nrseqcontrato, $nrorg);
            
            //$qtdHorasAgendadasTotais = 0;
            //$qtdHorasTrabalhadasTotais = 0;
            //$qtdHorasFaltaTotais = 0;
            //$qtdHorasAtrasoTotais = 0;
            //$qtdHorasExTotais = 0;
            $valorNovoVencimento = 0;
            $count = 0;
            
            foreach ($agendamentos as $agend) {
                $qtdHorasTrabalhadasRec = 0;
                $qtdHorasFaltaRec = 0;
                $qtdHorasAtrasoRec = 0;
                $qtdHorasEx50Rec = 0;
                $qtdHorasEx50RecSemFator = 0;
                $qtdHorasEx70Rec = 0;
                $qtdHorasEx70RecSemFator = 0;
                $qtdHorasEx100Rec = 0;
                $qtdHorasEx100RecSemFator = 0;
                $qtdHorasExTotaisRec = 0;
                $qtdHorasExTotaisRecSemFator = 0;
                
                $pessoa               = $agend["NRPARCNEPESSOA"];
                $nrItem               = $agend["NRITEMOCONTRATO"];
                $qtdHorasAgendadasRec = (float)$agend["QTHORAS"];
                $qtdDiasAgendadosRec  = (int)$agend["QTDIAS"];
                
                //$qtdHorasAgendadasTotais += $qtdHorasAgendadasRec;
                
                //$horasPorDia = $qtdHorasAgendadasRec / $qtdDiasAgendadosRec;
                
                //Verifica se tem eventos importados do HCM
                $eventosHCM = $this->validaTabelaTemporaria($nrvencprevcont, $nrseqcontrato, $pessoa, $nrorg);
                if ($eventosHCM > 0) {
                    
                    //$tempoBase = $this->getTotalHorasAgendadasContratoHora($nrseqcontrato, $nrItem, $nrorg, $pessoa);
                    
                    //Horas Trabalhadas
                    $eventos = "10";
                    $qtdDiasEvento10 = 0;
                    
                    $horas = $this->getEventos($nrvencprevcont, $nrseqcontrato, $pessoa, $nrorg, $eventos);
                    
                    if (count($horas) > 0) {
                        $qtdDiasEvento10 = $horas[0]["VRREFFOLHA"];
                    }
                    
                    
                    //Atestados
                    $eventos = "810";
                    $qtAtestados = 0;
                    
                    $horas = $this->getEventos($nrvencprevcont, $nrseqcontrato, $pessoa, $nrorg, $eventos);
                    
                    if (count($horas) > 0) {
                        $qtAtestados = $horas[0]["VRREFFOLHA"];
                        
                        $qtdDiasEvento10 = $qtdDiasEvento10+$qtAtestados;
                    }
                    
                    //$qtdHorasTrabalhadasRec = $qtdHorasAgendadasRec;
                    /*$eventos = "10_20";
                    $horas = $this->getEventos($nrvencprevcont, $nrseqcontrato, $pessoa, $nrorg, $eventos);
                    
                    foreach ($horas as $hora) {
                        if ($hora["NREVENTO"] == '10') {
                            $qtdHorasTrabalhadasRec = $hora["VRREFFOLHA"] * $horasPorDia;
                        }
                        else{
                           $qtdHorasTrabalhadasRec = $hora["VRREFFOLHA"];  
                        }
                    }*/
                    
                    //$qtdHorasTrabalhadasTotais += $qtdHorasTrabalhadasRec;
                    
                    
                    //Horas de Falta  (Não é mais consideradp)
                   /* $eventos = "1040";
                    $qtdDiasFaltaRec = 0;
                    
                    $horas = $this->getEventos($nrvencprevcont, $nrseqcontrato, $pessoa, $nrorg, $eventos);
                    
                    if (count($horas) > 0) {
                        $qtdDiasFaltaRec = $horas[0]["VRREFFOLHA"];
                    }*/
                    
                    /*$eventos = "1040_1041";
                    $horas = $this->getEventos($nrvencprevcont, $nrseqcontrato, $pessoa, $nrorg, $eventos);
                    
                    foreach ($horas as $hora) {
                        if ($hora["NREVENTO"] == '1040') {
                            $qtdHorasFaltaRec = (int)$hora["VRREFFOLHA"] * $horasPorDia;
                        }
                        else{
                           $qtdHorasFaltaRec = (float)str_replace(",", ".", $hora["VRREFFOLHA"]);
                        }
                    }*/
                    //$qtdHorasFaltaTotais += $qtdHorasFaltaRec;
                    
                    
                    //Horas de Atraso
                    $qtdHorasAtrasoRec = 0;
                    /*$eventos = "1050";
                    $horas = $this->getEventos($nrvencprevcont, $nrseqcontrato, $pessoa, $nrorg, $eventos);
                    
                    foreach ($horas as $hora) {
                        $qtdHorasAtrasoRec = (float)str_replace(",", ".", $hora["VRREFFOLHA"]);
                    }*/
                    
                    //$qtdHorasAtrasoTotais += $qtdHorasAtrasoRec;
                    
                    
                    //Horas Extra 50
                    $eventos = "70_970";
                    $horaExtra50 = $this->getEventos($nrvencprevcont, $nrseqcontrato, $pessoa, $nrorg, $eventos);
                        
                    foreach ($horaExtra50 as $horaE50) {
                        if ($horaE50["NREVENTO"] == '970'){
                           $qtdHorasEx50RecSemFator = Utilities::converteHoraDecimal(str_replace(",", ".", $horaE50["VRREFFOLHA"]));
                           $qtdHorasEx50Rec = $qtdHorasEx50RecSemFator * 1.5;
                        }
                        else{
                           $qtdHorasEx50RecSemFator = (float)str_replace(",", ".", $horaE50["VRREFFOLHA"]);
                           $qtdHorasEx50Rec = $qtdHorasEx50RecSemFator * 1.5;
                        }
                    }
                    
                    
                    //Horas Extra 70
                    $eventos = "74_974";
                    $horaExtra70 = $this->getEventos($nrvencprevcont, $nrseqcontrato, $pessoa, $nrorg, $eventos);
                    
                    foreach ($horaExtra70 as $horaE70) {
                        if ($horaE70["NREVENTO"] == '974'){
                            $qtdHorasEx70RecSemFator = Utilities::converteHoraDecimal(str_replace(",", ".", $horaE70["VRREFFOLHA"]));
                            $qtdHorasEx70Rec = $qtdHorasEx70RecSemFator * 1.7;
                        }
                        else{ 
                           $qtdHorasEx70RecSemFator = (float)str_replace(",", ".", $horaE70["VRREFFOLHA"]);
                           $qtdHorasEx70Rec = $qtdHorasEx70RecSemFator * 1.7;
                        }
                    }
                    
                    
                    //Horas Extra 100
                    $eventos = "90_990";
                    $horaExtra100 = $this->getEventos($nrvencprevcont, $nrseqcontrato, $pessoa, $nrorg, $eventos);
                    
                    foreach ($horaExtra100 as $horaE100) {
                        if ($horaE100["NREVENTO"] == '990'){
                           $qtdHorasEx100RecSemFator = Utilities::converteHoraDecimal(str_replace(",", ".", $horaE100["VRREFFOLHA"]));
                           $qtdHorasEx100Rec = $qtdHorasEx100RecSemFator * 2;
                        }
                        else{ 
                           $qtdHorasEx100RecSemFator = (float)str_replace(",", ".", $horaE100["VRREFFOLHA"]);
                           $qtdHorasEx100Rec = $qtdHorasEx100RecSemFator * 2;
                        }
                    }
                    
                    
                    //Horas Extras Totais
                    $qtdHorasExTotaisRec = $qtdHorasEx50Rec + $qtdHorasEx70Rec + $qtdHorasEx100Rec;
                    $qtdHorasExTotaisRecSemFator = $qtdHorasEx50RecSemFator + $qtdHorasEx70RecSemFator + $qtdHorasEx100RecSemFator;
                    
                    
                    if ($count == 0 || $nrItem != $nrItemAux) {
                        $dadosItem = $this->getDadosItemVencimento($nrseqcontrato, $nrItem, $nrorg);
                        //$valorUnitarioItem = $dadosItem["VRUNITITEMCONT"];
                        $valorUnitarioItem = (float)$dadosItem["VRTOTITEMCONT"] / $dadosItem["QTITEMCONTRATO"];
                        
                        //$valorItem = (float)$dadosItem["VRVENCIMENTO"] / $dadosItem["QTITEMCONTRATO"];
                        
                        //zera o valor dos itens do vencimento
                        $this->limparVrItemVencimentoHoras($nrvencprevcont, $nrseqcontrato, $nrItem, $nrorg);
                    }
                    
                    $valorHoraRec = $valorUnitarioItem / 220;
                    //$horasPorDia = 220 / 30;
                    
                    $qtdDiasTrabalhadosRec = $qtdDiasEvento10;
                    $valorTrabalhado = $valorUnitarioItem / 30 * $qtdDiasTrabalhadosRec;
                    
                    
                    //$qtdHorasApontadasIt = $qtdHorasTrabalhadasRec - $qtdHorasFaltaRec - $qtdHorasAtrasoRec + $qtdHorasExTotaisRec;
                    //$valorNovoIt = $valorHoraRec * $qtdHorasApontadasIt;
                    
                    $count++;
                    $nrItemAux = $nrItem;
                    
                    $valorHorasEx50Rec = 0;
                    $valorHorasEx70Rec = 0;
                    $valorHorasEx100Rec = 0;
                    $valorHorasExtrasRec = 0;
                    
                    if ($qtdHorasEx50Rec > 0)
                        $valorHorasEx50Rec = $valorHoraRec * $qtdHorasEx50Rec;
                    
                    if ($qtdHorasEx70Rec > 0)
                        $valorHorasEx70Rec = $valorHoraRec * $qtdHorasEx70Rec;
                        
                    if ($qtdHorasEx100Rec > 0)
                        $valorHorasEx100Rec = $valorHoraRec * $qtdHorasEx100Rec;
                    
                    if ($qtdHorasExTotaisRec > 0)
                        $valorHorasExtrasRec = $valorHoraRec * $qtdHorasExTotaisRec;
                    
                    
                    $valorNovoIt = $valorTrabalhado + $valorHorasExtrasRec;
                    $valorNovoVencimento += $valorNovoIt;
                    
                    $this->atualizarValorItVencHora($valorNovoIt, $nrvencprevcont, $nrseqcontrato, $nrItem, $nrorg);
                    
                    //SALVAR NA TABELA DE RELATORIO DE MEDICAO DO FATURAMENTO
                    $this->salvarTabelaRelatorioMedidaFaturamento($nrseqcontrato, $nrItem, $nrvencprevcont, $pessoa, $qtdDiasEvento10, null, $qtdDiasTrabalhadosRec, $valorTrabalhado, $qtdHorasEx50RecSemFator, $valorHorasEx50Rec, $qtdHorasEx70RecSemFator, $valorHorasEx70Rec, $qtdHorasEx100RecSemFator, $valorHorasEx100Rec, $qtdHorasExTotaisRecSemFator, $valorHorasExtrasRec, $valorNovoIt, $nrorg);
                }
                else {
                    $nomePessoa = $this->getNomePesssoa($pessoa, $nrorg);
                    throw new \Exception("Operação bloqueada.<br/>Não existem eventos para o seguinte recurso no período de competência da parcela selecionada:<br/>" . $pessoa . " - " . $nomePessoa);
                }
            }
            
            if ($count > 0) {
                //$horasApontadasTotais = $qtdHorasTrabalhadasTotais - $qtdHorasFaltaTotais - $qtdHorasAtrasoTotais + $qtdHorasExTotais;
                //$valorNovo = $valorVencimento / $qtdHorasAgendadasTotais * $horasApontadasTotais;
                $this->atualizarValorTotalVencimento($valorNovoVencimento, $nrvencprevcont, $nrseqcontrato, $nrorg);
            }
            else {
                throw new \Exception("Operação bloqueada.<br/>Não existem eventos para os recursos agendados neste contrato.");
            }
        }
        else {
            //$this->atualizarValorTotalVencimento(0, $nrvencprevcont, $nrseqcontrato, $nrorg);
            //$this->atualizarValorTodosItensVencimento(0, $nrvencprevcont, $nrseqcontrato, $nrorg);
            
            throw new \Exception("Operação bloqueada.<br/>Não há agendamento de recurso para este contrato neste período de competência.");
        }
    }
    
    public function excluirTabelaRelatorioMedicaoFaturamento($nrseqcontrato, $nrvencprevcont, $nrorg) {
        $params = array(
            "NRSEQCONTRATO"  => $nrseqcontrato,
            "NRVENCPREVCONT" => $nrvencprevcont,
            "NRORG"          => $nrorg
        );
        
        $this->apuracaoFaturamentoFactory->excluirTabelaRelatorioMedicaoFaturamento($params);
    }
    
    public function atualizarValorTodosItensVencimento($vrvencimento, $nrvencprevcont, $nrseqcontrato, $nrorg) {
        $params = array(
            "VRVENCIMENTO"   => $vrvencimento,
            "NRVENCPREVCONT" => $nrvencprevcont,
            "NRSEQCONTRATO"  => $nrseqcontrato,
            "NRORG"          => $nrorg
        );
        
        $this->apuracaoFaturamentoFactory->atualizarValorTodosItensVencimento($params);
    }
    
    public function getNomePesssoa($nrparcnegocio, $nrorg) {
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->apuracaoFaturamentoFactory->getNomePesssoa($params);
        return $dataset[0]["NMPRINCIPALPARC"];
    }
    
    
    public function atualizarValorParcela($nrvencprevcont, $nrSeqContrato, $nrorg, $operador) {
        
        $params = array(
            "NRVENCPREVCONT"   => $nrvencprevcont,
            "NRSEQCONTRATO"    => $nrSeqContrato,
            "NRORG"            => $nrorg
        );
        
        $vrItensAux = $this->apuracaoFaturamentoFactory->recebeValorItens($params);
        
        $vrItens =  (count($vrItensAux) > 0) ? $vrItensAux[0]["VRITENS"] : null;
        
        if ($vrItens <> null){
            
            $params = array(
                "NRVENCPREVCONT"   => $nrvencprevcont,
                "NRSEQCONTRATO"    => $nrSeqContrato,
                "VRITENS"          => $vrItens,
                "NRORG"            => $nrorg,
                "CDOPERADOR"       => $operador
            );
            
            $this->apuracaoFaturamentoFactory->atualizarValorParcela($params);
        }
    }
    
    /**
    * Rotina de Geração de Nota Antecipada.
    * 
    * Gera uma parcela do tipo "Periódico" e gera a nota fical corresponente a essa NF
    */
    public function gerarNotaAntecipada($nrvencprevcont, $nrseqcontrato, $nrgrupofatura, $idfaturacontr, $cdmodpgto, $nrapuracao, $qtdiasvencimento, $dtinicompet, $dtfincompet, $dtvencimento, $vrvencimento, $dtemissao, $idaltusuario, $idsituvencprev, $nragrupfatur, $nrparcnegocio, $vrfaturado, $vrfaltafaturar, $vrnotafiscante,  $nrorg, $operador) {
        
        $this->connection->beginTransaction();
        try {
            /* 1 - Cria a parcela períodica para a geração da nota antecipada. */
            $novoCodigoVenc = $this->newCode->getCodeByTableName("FCT_VENCPREVCONT", $nrorg, 1, 10, true);
            
            $paramsCriaParc = array(
                "NRVENCPREVCONT"   => $novoCodigoVenc,
                "NRSEQCONTRATO"    => $nrseqcontrato,
                "NRGRUPOFATURA"    => $nrgrupofatura,
                "IDFATURACONTR"    => 'A',
                "CDMODPGTO"        => $cdmodpgto,
                "NRAPURACAO"       => $nrapuracao,
                "QTDIASVENCIMENTO" => $qtdiasvencimento,
                "DTINICOMPET"      => $dtinicompet,
                "DTFINCOMPET"      => $dtfincompet,
                "DTVENCIMENTO"     => $dtvencimento,
                "VRVENCIMENTO"     => $vrvencimento,
                "DTEMISSAO"        => $dtemissao,
                "IDALTUSUARIO"     => $idaltusuario,
                "IDSITUVENCPREV"   => 'P',
                "NRAGRUPFATUR"     => $nragrupfatur,
                "NRPARCNEGOCIO"    => $nrparcnegocio,
                "VRFATURADO"       => $vrfaturado,
                "VRFALTAFATURAR"   => $vrfaltafaturar,
                "NRORG"            => $nrorg,
                "OPERADOR"         => $operador
            );
            
            $this->apuracaoFaturamentoFactory->criaParcelaNotaAnte($paramsCriaParc);
            
            
            /* 2 - Cria os itens na nova parcela de faturamento.*/
            $paramsItens = array(
                "NRVENCPREVCONT" => $nrvencprevcont,
                "NRSEQCONTRATO"  => $nrseqcontrato,
                "NRORG"          => $nrorg
            );
            
            $itensParc = $this->apuracaoFaturamentoFactory->recebeDadosItens($paramsItens);
            
            $VrParcela = 0;
            $cont = 0;
            foreach ($itensParc as $itens){
                
                $novoCodigoItens = $this->newCode->getCodeByTableName("FCT_ITVENCPREVCONT", $nrorg, 1, 10, true);
                
                $paramsItNovo = array(
                    "NRITVENCPREVCONT"      => $novoCodigoItens,
                    "NRVENCPREVCONT"        => $novoCodigoVenc,
                    "NRSEQCONTRATO"         => $itens["NRSEQCONTRATO"],
                    "NRITEMOCONTRATO"       => $itens["NRITEMOCONTRATO"],
                    "IDFATURACONTR"         => $itens["IDFATURACONTR"],
                    "CDMODPGTO"             => $itens["CDMODPGTO"],
                    "QTDIASVENCIMENTO"      => $itens["QTDIASVENCIMENTO"],
                    "DTINICOMPET"           => $itens["DTINICOMPET"],
                    "DTFINCOMPET"           => $itens["DTFINCOMPET"],
                    "DTVENCIMENTO"          => $itens["DTVENCIMENTO"],
                    "DTEMISSAO"             => $itens["DTEMISSAO"],
                    "VRVENCIMENTO"          => $itens["VRTOTITEMCONT"],
                    "QTITEMCONTRATO"        => $itens["QTITEMCONTRATO"],
                    "CDTPMEDIDASERV"        => $itens["CDTPMEDIDASERV"],
                    "QTMEDIDACONTR"         => $itens["QTMEDIDACONTR"],
                    "VRUNITITEMCONT"        => $itens["VRUNITITEMCONT"],
                    "VRTOTITEMCONT"         => $itens["VRTOTITEMCONT"],
                    "IDALTUSUARIO"          => 'N',
                    "QTPARCELASFATU"        => $itens["QTPARCELASFATU"],
                    "NRDIAVENCPARCELASFATU" => $itens["NRDIAVENCPARCELASFATU"],
                    "IDSITUITVENCPREV"      => $itens["IDSITUITVENCPREV"],
                    "NRORG"                 => $nrorg,
                    "OPERADOR"              => $operador
                );
                
                $this->apuracaoFaturamentoFactory->criaItensParcelaNotaAnte($paramsItNovo);
                
                $VrParcela = $VrParcela+$itens["VRTOTITEMCONT"];
                $cont = $cont + 1;
        
            }
            
            if (!empty($vrnotafiscante)){
              
              $vritens = $vrnotafiscante / $cont;
              $VrParcela = $vrnotafiscante;
              
              $paramsEditVrItens = array(
                "VRVENCIMENTO"     => $vrnotafiscante,
                "NRVENCPREVCONT"   => $novoCodigoVenc,
                "NRSEQCONTRATO"    => $nrseqcontrato,
                "NRORG"            => $nrorg
              );
              
              $this->apuracaoFaturamentoFactory->atualizaValorItemParcelaNotaAnte($paramsEditVrItens);
                
            };
            
            /* 3 - Atualiza Valor da parcela */
            
            $paramsEditParc = array(
                "VALOR"          => $VrParcela,
                "NRVENCPREVCONT" => $novoCodigoVenc,
                "NRSEQCONTRATO"  => $nrseqcontrato,
                "NRORG"          => $nrorg
            );
            
            $this->apuracaoFaturamentoFactory->atualizaValorParcelaNotaAnte($paramsEditParc);
            
            
            /* 4 - Emite NF antecipada. */
            
            $contrato    = '_'.$nrseqcontrato.'_';
            $parceiro    = '_'.$nrparcnegocio.'_';
            $parcela     = '_'.$novoCodigoVenc.'_';
            $grupofatura = '_'.$nrgrupofatura.'_';
            
            $this->executarFctFaturaContrato($contrato, $parceiro, $dtinicompet, $dtfincompet, $parcela, $grupofatura, null, $nrorg, $operador); 
            
            /* 5 - Atualiza parcela original com o nr. da parcela de Nota Antecipada. */
            
            $paramsAtuParc = array(
                "NRVENCPREVCONTANTECIP" => $novoCodigoVenc,
                "OPERADOR"              => $operador,
                "NRORG"                 => $nrorg,
                "NRVENCPREVCONT"        => $nrvencprevcont,
                "NRSEQCONTRATO"         => $nrseqcontrato
            );
            
            $this->apuracaoFaturamentoFactory->atualizaParcelaOriNotaAnte($paramsAtuParc);
            
            $this->connection->commit();
        }
        catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
        
    }
    
    
    
    /*----------------------------------------
     | --- Nova rotina de  Faturamento   --- |
     ----------------------------------------*/
    
    /**
     * Executa a procedure "FCT_APURAR_FATURAMENTO_LISTA".
     */
    public function gerarFaturamentoApuracao($nrfaturacont, $nritfaturacont, $nrrateioitemvenc, $nrorg, $operador) {
        $procedure = new FctApurarFaturamentoLista($this->connection);
        $procedure->executar($nrfaturacont, $nritfaturacont, $nrrateioitemvenc, $nrorg, $operador);
    }

    
     /**
     * Executa a procedure "FCT_CANC_APURCAO_FATURAMENTO".
     */
    public function cancelarFaturamentoApuracao($nrseqcontrato, $dataini, $datafin, $nrfaturacont, $nrrateioitemvenc, $nrorg, $cdoperador, $nritfaturacont) {
        $procedure = new FctCancApuracaoFaturamento($this->connection);
        $procedure->executar($nrorg, $nrseqcontrato, $cdoperador, $dataini, $datafin, $nrfaturacont, $nrrateioitemvenc, $nritfaturacont);
    }
    
    /**
     * Anular a apuração da parcela.
     */
    public function anularApuracao($dsJustificativa, $nrfaturacont, $nrSeqContrato, $nrorg, $operador) {
        $params = array(
            "NRFATURACONT"  => $nrfaturacont,
            "NRORG"         => $nrorg
        );
        $dataset = $this->apuracaoFaturamentoFactory->getJustificativa($params);
        $justificativaAtual = $dataset[0]["DSJUSTIFICATIVA"];

        if (empty($justificativaAtual)) 
            $novaJustificativa = $dsJustificativa;
        else 
            $novaJustificativa = $dsJustificativa . "\n" . $justificativaAtual;

        $params = array(
            "DSJUSTIFICATIVA"  => $novaJustificativa,
            "NRFATURACONT"     => $nrfaturacont,
            "NRORG"            => $nrorg,
            "CDOPERADOR"       => $operador
        );
        
        $this->apuracaoFaturamentoFactory->anularApuracao($params);
    }

    /**
     * Editar descrição Nota Fiscal e Boleto
     */
    public function salvarWdgEditarDescNf($nrSeqContrato, $dsNota, $dsBoleto, $nrorg, $operador, $verificarAgrupamento) {

        $params = array(
            "NRSEQCONTRATO"    => $nrSeqContrato,
            "DSSERVORCANF"     => $dsNota,
            "DSSERVORCABOL"    => $dsBoleto,
            "NRORG"            => $nrorg,
            "CDOPERADOR"       => $operador
        );

// var_dump($verificarAgrupamento);
// die;
        if ($verificarAgrupamento == 0) {
            $this->apuracaoFaturamentoFactory->salvarWdgEditarDescNf($params);
        }elseif ($verificarAgrupamento > 0) {
            $this->apuracaoFaturamentoFactory->salvarWdgEditarDescNfAgrup($params);
        }
    }

    public function verificarAgrupamento($nrSeqContrato){
        $params = array(
            "NRSEQCONTRATO" => $nrSeqContrato
        );
        
        $resultado = $this->apuracaoFaturamentoFactory->verificarAgrupamento($params);
        return $resultado; 
    }
    
    /**
     * A função "FatuManual" Realiza o Faturamento MANUAL das parcelas.
     */
    public function FatuManual($nrfaturacont, $nrSeqContrato, $nrParcFatura, $vrVencimento, $justificativa, $nrLanctoNF, $cdFilial, $cdEmpresa, $vrNotaFisc, $nrorg, $operador) {
       // 1 - Primeiro, gera o registro de Apuração
       
       $novoCodigo = $this->newCode->getCodeByTableName("FCT_APURACAO", $nrorg, 1, 10, true);
       $params = array(
            "NRAPURACAO"       => $novoCodigo,
            "NRSEQCONTRATO"    => null,
            "NRPARCFATURA"     => $nrParcFatura,
            "NRGRUPOFATURA"    => null,
            "CDEMPRESA"        => $cdEmpresa,
            "CDFILIAL"         => $cdFilial,
            "NRLANCTONF"       => $nrLanctoNF,
            "VRNOTAFISC"       => $vrNotaFisc,
            "VRVENCIMENTO"     => $vrVencimento,
            "NRORG"            => $nrorg,
            "CDOPERADOR"       => $operador
        );
        
        $this->apuracaoFaturamentoFactory->criaApuracaoManual($params);
        
        // 2- insere o relacionamento de apuracao com faturamento
        $this->inserirRelacApuracaoFatura($nrfaturacont, $novoCodigo, $nrorg, $operador);
        
        // 3 - Atualiza a Parcela
        $params = array(
            "NRFATURACONT"  => $nrfaturacont,
            "NRORG"         => $nrorg
        );
        $dataset = $this->apuracaoFaturamentoFactory->getJustificativa($params);
        $justificativaAtual = $dataset[0]["DSJUSTIFICATIVA"];

        if (empty($justificativaAtual))
            $novaJustificativa = $justificativa;
        else 
            $novaJustificativa = $justificativa . "\n" . $justificativaAtual;
            
        $params = array(
            "NRAPURACAO"       => null,//$novoCodigo,
            "DSJUSTIFICATIVA"  => $novaJustificativa,       
            "VRNOTAFISC"       => $vrNotaFisc,
            "NRFATURACONT"     => $nrfaturacont,
            "NRORG"            => $nrorg,
            "CDOPERADOR"       => $operador
        );
        
        $this->apuracaoFaturamentoFactory->atualizaParcelaApuracaoManual($params);
    }

    /**
     * Insere uma apuração na tabela de relacionamento com a parcela
    **/
    public function inserirRelacApuracaoFatura($nrfaturacont, $nrapuracao, $nrorg, $operador){
        $nrapurfatura = $this->newCode->getCodeByTableName("FCT_RELACAPURACAOFATURA", $nrorg, 1, 10, true);
        $params = array(
            "NRAPURACAOFATURA" => $nrapurfatura,
            "NRAPURACAO"       => $nrapuracao,
            "NRFATURACONT"     => $nrfaturacont,
            "NRORG"            => $nrorg,
            "CDOPERADOR"       => $operador
        );
        $this->apuracaoFaturamentoFactory->inserirRelacApuracaoFatura($params);
    }
    
    /**
     * A função "validaQtItens" valida a quantidade de itens da parcela.
     */
    public function validaQtItens($nrfaturacont, $nrSeqContrato, $nrorg) {
        $params = array(
            "NRFATURACONT"     => $nrfaturacont,
            "NRORG"            => $nrorg
        );
        
        $dataset = $this->apuracaoFaturamentoFactory->validaQtItens($params);
        return (count($dataset) > 0) ? $dataset[0]["VALIDA"] : null;
    }
    
    /**
     * A função "desmembrarParcela" Realiza o desmembramento dos itens das parcelas.
     */
    public function desmembrarParcela($nrfaturacont, $nrSeqContrato, $itDesmembramento, $idFormaDesmembramento, $dsJustificativa, $nrorg, $operador) {
        $this->connection->beginTransaction();
        try {
            $novoNrFaturaCont = null;
            $params = array(
                "NRFATURACONT"     => $nrfaturacont,
                "NRORG"            => $nrorg
            );
            
            $dadosParcela = $this->apuracaoFaturamentoFactory->getDadosParcela($params);
            $dadosParcela = (count($dadosParcela) > 0) ? $dadosParcela[0] : null;
            if ($dadosParcela != null){
                $justificativaAtual = $dadosParcela["DSJUSTIFICATIVA"];
                $novaJustificativa  = $dsJustificativa . ((empty($justificativaAtual)) ? "" : "\n" . $justificativaAtual);
                
                foreach ($itDesmembramento as $item) {
                    $params = array(
                        "NRITFATURACONT"   => $item,
                        "NRFATURACONT"     => $nrfaturacont,
                        "NRORG"            => $nrorg
                    );
                    
                    $dadosItem = $this->apuracaoFaturamentoFactory->getDadosItem($params);
                    $dadosItem = (count($dadosItem) > 0) ? $dadosItem[0] : null;
                    
                    if ($dadosItem != null) {
                        $idAcao = $idFormaDesmembramento == '1' ? "DESAGRUPA" : "AGRUPA";
                        /*Gerar uma parcela para cada item selecionado*/
                        if ($idFormaDesmembramento == '1' || $novoNrFaturaCont == null) {
                            /* 1 -  Insere a Parcela*/
                            $dadosParcela["VRVENCIMENTO"] = $dadosItem["VRVENCIMENTO"];
                            $novoNrFaturaCont = $this->inserirParcela($dadosParcela, $dadosItem["NRSEQCONTRATO"], $nrorg, $operador, $novaJustificativa);
                            $this->inserirRelacAgrupContFat($dadosItem["NRSEQCONTRATO"], $novoNrFaturaCont, $nrorg, $operador, $idAcao, $dadosParcela["NRMEDICAOFATURA"]);
                            //$this->alterarRelacContratosFatura($nrfaturacont, $dadosItem["NRSEQCONTRATO"], "DESAGRUPA", $nrorg, $operador, $novoNrFaturaCont);
                            /* 2 - Insere o Item */
                            $this->inserirItemParcela(array($dadosItem), $nrfaturacont, $nrorg, $novoNrFaturaCont, $operador, $novaJustificativa);
                        } else {
                            /*1 - Insere o novo item*/
                            $this->inserirItemParcela(array($dadosItem), $nrfaturacont, $nrorg, $novoNrFaturaCont, $operador, $novaJustificativa);
                            $this->inserirRelacAgrupContFat($dadosItem["NRSEQCONTRATO"], $novoNrFaturaCont, $nrorg, $operador, $idAcao, $dadosParcela["NRMEDICAOFATURA"]);
    
                            /* 2 - Atualiza o valor da nova parcela criada */
                            $params = array(
                                "VRVENCIMENTO" => $dadosItem["VRVENCIMENTO"], 
                                "NRFATURACONT" => $novoNrFaturaCont, 
                                "NRORG"        => $nrorg
                            );
                            
                            $this->apuracaoFaturamentoFactory->atualizaValorParcelaNova($params);    
                        }
                    
                        /* Deleta o item da parcela de Origem */
                        $params = array(
                            "NRITFATURACONT"   => $item,
                            "NRFATURACONT"     => $nrfaturacont,
                            "NRSEQCONTRATO"    => $dadosItem["NRSEQCONTRATO"],
                            "NRORG"            => $nrorg
                        );
                        $this->apuracaoFaturamentoFactory->DeletaItemParcelaOrigem($params);    
                        
                        /* Atualiza a parcela de origem */
                        $params = array(
                            "VRVENCIMENTO"     => $dadosItem["VRVENCIMENTO"],
                            "DSJUSTIFICATIVA"  => $novaJustificativa,
                            "NRFATURACONT"     => $nrfaturacont,
                            "NRORG"            => $nrorg
                        );
                        $this->apuracaoFaturamentoFactory->atualizaValorParcelaOrigem($params);
                    } 
                }
            }
            
            $this->connection->commit();
        } catch(\Exception $e){
            $this->connection->rollback();
            throw $e;
        }
    }
    /**
     * Valida as parcelas selecionadas para serem agrupadas 
    **/
    public function agruparParcelasParaFaturamento($nrfaturaconts, $idAcao, $nrorg, $operador){
        $this->connection->beginTransaction();
        try {
            $nrfaturaconts = implode(",", $nrfaturaconts);
            $infoParcelas = $this->apuracaoFaturamentoFactory->validarParcelasParaAgrupamento($nrfaturaconts, $nrorg);
            if(count($infoParcelas) !== 1){
                throw new \Exception("Operação bloqueada!<br>Não é possível agrupar as parcelas selecionadas. Verifique se as parcelas estão pendentes e os parceiros de negócio, as filiais, os grupos de faturamento, os tipos de faturamento e o período da competência são iguais.");
            } else {
                $vrTotalParcela  = 0;
                $infoParcelas    = $infoParcelas[0];
                
                $newnrfaturacont = $this->inserirParcela($infoParcelas, null, $nrorg, $operador, null);
                $this->alterarRelacContratosFatura($nrfaturaconts, null, $idAcao, $nrorg, $operador, $newnrfaturacont);
                $this->inserirItemParcela(null, $nrfaturaconts, $nrorg, $newnrfaturacont, $operador, null);
                
                foreach(explode(",", $nrfaturaconts) as $nrfaturacont) {
                    $params = array(
                        "NRITFATURACONT"   => null,
                        "NRSEQCONTRATO"    => null,
                        "NRFATURACONT"     => $nrfaturacont,
                        "NRORG"            => $nrorg
                    );
                    
                    $this->apuracaoFaturamentoFactory->DeletaItemParcelaOrigem($params); 
                }
                
                $this->apuracaoFaturamentoFactory->deletarParcelaFatura($nrfaturaconts, $nrorg);
                $this->connection->commit();
            }
        } catch(\Exception $e){
            $this->connection->rollback();
            throw $e;
        }
    }
    
    /**
     * Insere na tabela FCT_RELACAGRUPCONTFAT os contratos e medições ligados as 
     * parcelas agrupadas
    **/
    public function alterarRelacContratosFatura($nrfaturaconts, $nrseqcontrato, $idAcao, $nrorg, $operador, $newnrfaturacont){
        try{
            $paramsRelac = array(
                "NRSEQCONTRATO" => $nrseqcontrato,
                "NRORG"         => $nrorg
            );
            $relacParcelas = $this->apuracaoFaturamentoFactory->getInformacoesRelacParcelaCont($nrfaturaconts, $paramsRelac);
            foreach($relacParcelas as $relacParcela){
                $this->inserirRelacAgrupContFat($relacParcela["NRSEQCONTRATO"], $newnrfaturacont, $nrorg, $operador, $idAcao, $relacParcela["NRMEDICAOFATURA"]);
            }
            
            $this->apuracaoFaturamentoFactory->deletarRelacAgrupamentoContFat($nrfaturaconts, $paramsRelac);
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function inserirRelacAgrupContFat($nrseqcontrato, $nrfaturacont, $nrorg, $operador, $idAcao, $nrmedicaofatura) {
        $nragrupcontfat = $this->newCode->getCodeByTableName("FCT_RELACAGRUPCONTFAT", $nrorg, 1, 10, true);
        $params = array(
            "NRAGRUPCONTFAT"  => $nragrupcontfat,
            "NRSEQCONTRATO"   => $nrseqcontrato,
            "NRORG"           => $nrorg,
            "OPERADOR"        => $operador, 
            "NRFATURACONT"    => $nrfaturacont,
            "IDPARCAGRUP"     => ($idAcao == "AGRUPA") ? "S" : "N",
            "NRMEDICAOFATURA" => $nrmedicaofatura
        );
        $this->apuracaoFaturamentoFactory->salvarRelacAgrupContFat($params);
    }
    
    /**
     * Manda inserir os itens da parcela
    **/
    public function inserirItemParcela($itensFat, $nrfaturaconts, $nrorg, $newnrfaturacont, $operador, $dsJustificativa){
        if($itensFat == null) {
            $params = array(
                "NRORG"         => $nrorg,
                "NRSEQCONTRATO" => null
            );
            $itensFat = $this->apuracaoFaturamentoFactory->getItemsFatura($params, $nrfaturaconts);
        }
    
        foreach($itensFat as $itemFat){
            $novoNrFaturaItCont = $this->newCode->getCodeByTableName("FCT_ITFATURACONT", $nrorg, 1, 10, true);
            $params = array(
                "NRITFATURACONT"     => $novoNrFaturaItCont, 
                "NRFATURACONT"       => $newnrfaturacont, 
                "NRSEQCONTRATO"      => $itemFat["NRSEQCONTRATO"], 
                "NRITEMOCONTRATO"    => $itemFat["NRITEMOCONTRATO"], 
                "CDPRODFATURA"       => $itemFat["CDPRODFATURA"], 
                "IDSITUACAO"         => $itemFat["IDSITUACAO"], 
                "VRVENCIMENTO"       => $itemFat["VRVENCIMENTO"], 
                "VRFALTAFATURAR"     => $itemFat["VRVENCIMENTO"], 
                "IDFORMAFATURA"      => $itemFat["IDFORMAFATURA"], 
                "NRAPURACAO"         => $itemFat["NRAPURACAO"],
                "DSJUSTIFICATIVA"    => $dsJustificativa,
                "VRFATURADO"         => 0, 
                "NRORG"              => $nrorg,
                "OPERADOR"           => $operador
            );
            
            $this->apuracaoFaturamentoFactory->insereItemDesmembramento($params);
        }
    }
    /**
     * Cria n outras parcelas para efetuar o desagrupamento de contratos 
    **/
    public function desagruparParcelasFaturamento($nrfaturacont, $nrseqcontratos, $idAcao, $nrorg, $operador){
        $this->connection->beginTransaction();
        try {
            $params = array(
                "NRFATURACONT" => $nrfaturacont,
                "NRORG"        => $nrorg
            );
            $dadosParcela = $this->apuracaoFaturamentoFactory->getDadosParcela($params);
            if(!isset($dadosParcela)){
                throw new \Exception("Operação bloqueada!<br>Falha ao efetuar o desmembramento de contratos.");
            } else {
                $vrvencimentoold = 0;
                $dadosParcela    = $dadosParcela[0];
                foreach($nrseqcontratos as $nrseqcontrato){
                    $vrTotalParcela = 0;
                    $params = array(
                        "NRORG"         => $nrorg,
                        "NRSEQCONTRATO" => $nrseqcontrato
                    );
                    
                    $dadosItem = $this->apuracaoFaturamentoFactory->getItemsFatura($params, $nrfaturacont);
                    foreach($dadosItem as $dadoItem)
                        $vrTotalParcela = $vrTotalParcela + $dadoItem["VRVENCIMENTO"];
                    
                    $dadosParcela["VRVENCIMENTO"] = $vrTotalParcela;    
                    $vrvencimentoold = $vrvencimentoold + $vrTotalParcela;
                    $newnrfaturacont = $this->inserirParcela($dadosParcela, $nrseqcontrato, $nrorg, $operador, null);
                    $this->alterarRelacContratosFatura($nrfaturacont, $nrseqcontrato, $idAcao, $nrorg, $operador, $newnrfaturacont);
                    
                    foreach($dadosItem as $dadoItem)
                        $this->inserirItensFatura($dadoItem, $nrseqcontrato, $newnrfaturacont, null, $nrorg, $operador);
                }
                
                $this->validarQuantidadeContratosParcela($nrfaturacont, $nrorg, $operador, $vrvencimentoold, $nrseqcontratos);
                $this->connection->commit();
            }
        } catch(\Exception $e){
            $this->connection->rollback();
            throw $e;
        }
    }
    /**
     * Pega as informações de uma parcela e insere criando outra similar 
    **/
    public function inserirParcela($dadosParcela, $contrato, $nrorg, $operador, $dsJustificativa){
        $newnrfaturacont = $this->newCode->getCodeByTableName("FCT_FATURACONT", $nrorg, 1, 10, true);
        
        $nrparcela = null;
        if (!empty($contrato)) {
            $params = array(
                "NRSEQCONTRATO" => $contrato,
                "NRORG"         => $nrorg
            );
            $dataset = $this->apuracaoFaturamentoFactory->getQuantidadeParcelasContrato($params);
            $nrparcela = (int)$dataset[0]["CONTADOR"] + 1;
        }
        
        $params = array(
            "NRFATURACONT"       => $newnrfaturacont,
            "NRSEQCONTRATO"      => null, //$contrato,
            "NRGRUPOFATURA"      => $dadosParcela["NRGRUPOFATURA"],
            "CDFILIALFATURA"     => $dadosParcela["CDFILIALFATURA"],
            "NRPARCFATURA"       => $dadosParcela["NRPARCFATURA"],
            "IDTPFATURA"         => $dadosParcela["IDTPFATURA"],
            "IDSITUACAO"         => $dadosParcela["IDSITUACAO"],
            "DTINICOMPET"        => $dadosParcela["DTINICOMPET"],
            "DTFINCOMPET"        => $dadosParcela["DTFINCOMPET"],
            "DTLIMITEAPU"        => $dadosParcela["DTLIMITEAPU"],
            "DTVENCIMENTO"       => $dadosParcela["DTVENCIMENTO"],
            "NRDIASVENCIMENTO"   => $dadosParcela["NRDIASVENCIMENTO"],
            "QTDIASFATUVENC"     => $dadosParcela["QTDIASFATUVENC"],
            "VRVENCIMENTO"       => $dadosParcela["VRVENCIMENTO"],
            "VRFALTAFATURAR"     => $dadosParcela["VRVENCIMENTO"],
            "IDFORMAFATURA"      => $dadosParcela["IDFORMAFATURA"],
            "DSJUSTIFICATIVA"    => $dsJustificativa,
            "NRAPURACAO"         => $dadosParcela["NRAPURACAO"],
            "NRMEDICAOFATURA"    => $dadosParcela["NRMEDICAOFATURA"],
            "CDTIPOOPERFATURA"   => $dadosParcela["CDTIPOOPERFATURA"],
            "CDSERIENF"          => $dadosParcela["CDSERIENF"],
            "CDEMPRESA"          => $dadosParcela["CDEMPRESA"],
            "CDINSCESTA"         => $dadosParcela["CDINSCESTA"],
            "CDINSCMUNI"         => $dadosParcela["CDINSCMUNI"],
            "VRFATURADO"         => 0,
            "NRORG"              => $nrorg,
            "OPERADOR"           => $operador,
            "NRPARCELA"          => $nrparcela
        );
        $this->apuracaoFaturamentoFactory->insereParcelaDesmembramento($params);
        
        /*$newagrupfat = $this->newCode->getCodeByTableName("FCT_RELACAGRUPCONTFAT", $nrorg, 1, 10, true);
        $params = array(
            "NRAGRUPCONTFAT"  => $newagrupfat,
            "NRSEQCONTRATO"   => $contrato,
            "NRORG"           => $nrorg,
            "OPERADOR"        => $operador,
            "NRFATURACONT"    => $newnrfaturacont,
            "IDPARCAGRUP"     => "N",
            "NRMEDICAOFATURA" => $dadosParcela["NRMEDICAOFATURA"]
        );
        $this->apuracaoFaturamentoFactory->salvarRelacAgrupContFat($params);*/
                
        return $newnrfaturacont;
    }
    /**
     * Insere os itens das respectivas novas parcelas criadas
    **/
    public function inserirItensFatura($dadosItem, $contrato, $newnrfaturacont, $dsJustificativa, $nrorg, $operador){
        $newnrfaturaitem = $this->newCode->getCodeByTableName("FCT_ITFATURACONT", $nrorg, 1, 10, true);
        $params = array(
            "NRITFATURACONT"     => $newnrfaturaitem, 
            "NRFATURACONT"       => $newnrfaturacont, 
            "NRSEQCONTRATO"      => $contrato, 
            "DSJUSTIFICATIVA"    => $dsJustificativa,
            "VRFATURADO"         => 0, 
            "NRITEMOCONTRATO"    => $dadosItem["NRITEMOCONTRATO"], 
            "CDPRODFATURA"       => $dadosItem["CDPRODFATURA"], 
            "IDSITUACAO"         => $dadosItem["IDSITUACAO"], 
            "VRVENCIMENTO"       => $dadosItem["VRVENCIMENTO"], 
            "VRFALTAFATURAR"     => $dadosItem["VRVENCIMENTO"], 
            "IDFORMAFATURA"      => $dadosItem["IDFORMAFATURA"], 
            "NRAPURACAO"         => $dadosItem["NRAPURACAO"],
            "NRORG"              => $nrorg,
            "OPERADOR"           => $operador
        );
        
        $this->apuracaoFaturamentoFactory->insereItemDesmembramento($params);
    }
    /**
     * Verifica se a parcela ainda é um agrupamento
    **/
    public function validarQuantidadeContratosParcela($nrfaturacont, $nrorg, $operador, $vrvencimentoold, $nrseqcontratos){
        $params = array(
            "NRSEQCONTRATO" => null,
            "NRORG"         => $nrorg
        );
        $relacParcelas = $this->apuracaoFaturamentoFactory->getInformacoesRelacParcelaCont($nrfaturacont, $params);
        if(count($relacParcelas) == 0) {
            $this->apuracaoFaturamentoFactory->deletarParcelaFatura($nrfaturacont, $nrorg);
        } else {
            if(count($relacParcelas) == 1) {
                $params = array(
                    "NRFATURACONT" => $nrfaturacont, 
                    "NRORG"        => $nrorg, 
                    "CDOPERADOR"   => $operador
                );
                $this->apuracaoFaturamentoFactory->alterarInfoAgrupamentoParcela($params);
            }
            
            $params = array(
                "NRFATURACONT"    => $nrfaturacont, 
                "NRORG"           => $nrorg, 
                "VRVENCIMENTO"    => $vrvencimentoold, 
                "DSJUSTIFICATIVA" => null
            );
            $this->apuracaoFaturamentoFactory->atualizaValorParcelaOrigem($params);
        }

        foreach($nrseqcontratos as $nrseqcontrato) {
            $params = array(
                "NRITFATURACONT"   => null,
                "NRFATURACONT"     => $nrfaturacont,
                "NRSEQCONTRATO"    => $nrseqcontrato,
                "NRORG"            => $nrorg
            );
            
            $this->apuracaoFaturamentoFactory->DeletaItemParcelaOrigem($params); 
        }
    }
    /**
     * Desagrupa um contrato de todas as parcelas ainda pendentes de um faturamento
    **/
    public function retirarContratoAgrupamentoFat($nragrupamento, $nrseqcontratos, $nrorg, $operador){
        $params = array(
            "NRAGRUPAMENTO" => $nragrupamento,
            "NRORG"         => $nrorg
        );
        
        $deletarAgrupamento = false;
        if($nrseqcontratos == null) {
            $deletarAgrupamento = true;
            $nrseqcontratos     = $this->apuracaoFaturamentoFactory->getContratosAgrupamento($params);
        }
        
        if(count($nrseqcontratos) > 0) {
            $contratos     = implode(",", $nrseqcontratos);
            $nrfaturaconts = $this->apuracaoFaturamentoFactory->getParcelasAgrupamentoCont($params, $contratos);
            if(count($nrfaturaconts) > 0){
                foreach($nrfaturaconts as $nrfaturacont)
                    $this->desagruparParcelasFaturamento($nrfaturacont["NRFATURACONT"], $nrseqcontratos, "DESAGRUPA", $nrorg, $operador);
            }
            $this->apuracaoFaturamentoFactory->deletarContratoAgrupamento($contratos, $params);
        }
        
        if($deletarAgrupamento) {
            $this->apuracaoFaturamentoFactory->deletarEscoposAgrupamento($params);
            $this->apuracaoFaturamentoFactory->deletarAgrupamento($params);
        }
    }
    /*
     * Valida se há informações na tabela de relatório 
    **/
    public function getPossuiRelatorioEspelho($nrseqcontrato, $nrvencprevcont, $nrorg){
        $params = array(
            "NRSEQCONTRATO"  => $nrseqcontrato,
            "NRVENCPREVCONT" => $nrvencprevcont,
            "NRORG"          => $nrorg
        );
        return $this->apuracaoFaturamentoFactory->getPossuiRelatorioEspelho($params)[0]["POSSUIRELAT"] > 0;
    }
    
    
    public function validarAgrupamentoFat($nrseqcontrato, $nrparcfatura, $nrcontratofct, $nrfaturacont, $dtinicio, $dtfim, $nrorg){
        
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRPARCFATURA"  => $nrparcfatura,
            "NRFATURACONT"  => $nrfaturacont,
            "DTINICIO"      => $dtinicio,
            "DTFIM"         => $dtfim,
            "NRORG"         => $nrorg
        );
        
        $resultado = $this->apuracaoFaturamentoFactory->validarAgrupamentoFat($params);
        
        return $resultado; 
    }
    
    
    
    
    
    public function validarAgrupamentoFatMarcados($vencimentosMarcados, $nrorg){
        
        $erros = 0;
        $contratos = "";
        
        foreach($vencimentosMarcados as $vencimentos) {
            
            $params = array(
                "NRSEQCONTRATO" => '_'.$vencimentos["NRSEQCONTRATO"].'_', 
                "NRPARCFATURA"  => $vencimentos["NRPARCFATURA"], 
                "NRFATURACONT"  => $vencimentos["NRFATURACONT"], 
                "DTINICIO"      => $vencimentos["DTINICOMPET"], 
                "DTFIM"         => $vencimentos["DTFINCOMPET"], 
                "NRORG"         => $nrorg
            );
            
            $resultado = $this->apuracaoFaturamentoFactory->validarAgrupamentoFat($params);
            
            if (!empty($resultado)) {
                
                $erros = $erros+1;
                
                foreach($resultado as $dados){
                    
                    $contratos = $contratos."- ".$dados['NRCONTRATOFCT']."<br>";
                }
    
            }
        }    
        
        if ($erros == 0){
            
            $contratos = "OK"; 
        }
        
        return $contratos; 
    }
    
    /**
     * Valida se existem contratos para o período que não tiveram reajuste.
    **/
    public function validarReajuste($nrorg){
        
       /* Trecho comentato pegava o valor do menmônico para definir se deveria ter a trava de reajsute de contrato, o mesmo foi substituido pelo campo na prarametrização sm.
      
        $validaReajuste = $this->validaReajusteFatu;
        
        if ($validaReajuste == true){*/
        
        $params = array(
                "NRORG"  => $nrorg
            );
            
        $validaReajuste = $this->apuracaoFaturamentoFactory->validaReajusteFatu($params);
        
        if ($validaReajuste[0]['IDBLOQAPURFATSEMREAJ'] == 'S'){
            
            $params = array(
                "NRORG"  => $nrorg
            );
            
            $this->apuracaoFaturamentoFactory->deletaTempContPendenciaReaj($params);
            
            $resultado = $this->apuracaoFaturamentoFactory->validarReajuste($params);
            
            foreach($resultado as $result){
                
                $paramsIns =  array(
                    
                    "NRSEQCONTRATO" => $result["NRSEQCONTRATO"], 
                    "NRCONTRATOFCT" => $result["NRCONTRATOFCT"], 
                    "NRPARCNEGOCIO" => $result["NRPARCNEGOCIO"], 
                    "NMPRINCIPALPARC" => $result["NMPRINCIPALPARC"],
                    "DTINICIOCONTRATO" => $result["DTINICIO"], 
                    "DTFIMCONTRATO" => $result["DTFINAL"], 
                    "DSREFERENCIACONT" => $result["DSREFERENCIACONT"], 
                    "NRORG" => $nrorg
                    
                );
                
                $this->apuracaoFaturamentoFactory->insereTempContPendenciaReaj($paramsIns);
            }
        }
        else {
            
            $params = array(
                "NRORG"  => $nrorg
            );
            
            $this->apuracaoFaturamentoFactory->deletaTempContPendenciaReaj($params);
            
            $resultado = array();
        }
        
        return $resultado; 
    }
    
    /**
     * Atualiza as situação das parcelas zeradas para faturada
    **/
    public function alterarStatusParcelasZeradas($nrfaturacont, $nrorg, $cdoperador, $situacao) {
        $params = array(
            "NRFATURACONT" => $nrfaturacont,
            "IDSITUACAO"   => $situacao,
            "CDOPERULTATU" => $cdoperador,
            "NRORG"        => $nrorg
        );
        
        $this->apuracaoFaturamentoFactory->alterarSituacaoParcela($params);
        $this->apuracaoFaturamentoFactory->alterarSituacaoItens($params);
        $this->apuracaoFaturamentoFactory->alterarSituacaoRateio($params);
    }
    
    /**
     * Altera na tabela de nota fiscal a situação e data de saída da nota
    **/
    public function alterarDataSaidaDaNotaFiscal($row, $nrorg, $cdoperador) {
        $notasFiscais = $row["NOTASFISCAIS"];
        foreach($notasFiscais as $notaFiscal) {
            $params = array(
                "IDSTATUS"     => "D",
                "DTENTRSAID"   => $row["NEWDTSAIDANF"],
                "CDOPERULTATU" => $cdoperador,
                "CDFILIAL"     => $notaFiscal["CDFILIAL"],
                "NRLANCTONF"   => $notaFiscal["NRLANCTONF"],
                "NRORG"        => $nrorg
            );
            
            $this->apuracaoFaturamentoFactory->alterarDataSaidaDaNotaFiscal($params);
        }
    }
}

?>