/*
--Tecnisa - Tecnologia em Informtica
--
--Nome: EXPORTA_NUTRIENTES
--
--Descrio:
--  Procedure usada no programa GER04001 - Cpia de nutrientes.
--  Copia os nutrientes de um produtos para vrios outros,
--  sobrepondo os atuais ou acrescentando os novos.
--Criao:
--04/04/2001 - Dmaso
--
--@C:\Web_406\Sql\SqlServer\7150_PLASTP_EXPORTA_NUTRIENTES.SQL

Parmetros:
  P_CDPRODORIG -- Cdigo do produto de origem
  P_LISTANUTR -- Lista dos nutrientes de origem (que sero copiados)
  P_LISTAPROD -- Lista dos produtos de destino (que sero modificados)
  P_OPERACAO -- 'A' --> ACRESCENTAR
             -- 'S' --> SOBREPOR

*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
                    WHERE
	        O.NAME = 'EXPORTA_NUTRIENTES'  AND
	        TYPE = 'P' 			   AND
	        U.UID = O.UID             	   AND
	        U.NAME = USER)
  DROP PROCEDURE EXPORTA_NUTRIENTES
  GO

CREATE PROCEDURE EXPORTA_NUTRIENTES
 @P_CDPRODORIG VARCHAR(20),
 @P_LISTANUTR  VARCHAR(2000),
 @P_LISTAPROD  VARCHAR(2000),
 @P_OPERACAO   VARCHAR(1)

AS
   DECLARE @C_PRODDEST_CDPRODUTO VARCHAR(10);
   DECLARE @C_NUTRIENTES_CDPRODUTO VARCHAR(10);
   DECLARE @C_NUTRIENTES_CDNUTRIENTE VARCHAR(3);
   DECLARE @C_NUTRIENTES_QTNUTRPROD NUMERIC(12,5);
   DECLARE @C_NUTRIENTES_DSFONTNUTR VARCHAR(40);
BEGIN

   DECLARE C_PRODDEST CURSOR FOR
      /* Seleciona os produtos de destino */
   SELECT
      DISTINCT CDPRODUTO
   FROM
      PRODUTO, LISTA
   WHERE
      (CDARVPROD <> @P_CDPRODORIG  ) AND
      (CDPRODUTO = CDPRODESTO      ) AND
      (CDARVPROD =  LISTA.CDLISTA  ) AND
      (LISTA.NRLISTA = @P_LISTAPROD);

   OPEN C_PRODDEST;
   FETCH NEXT FROM C_PRODDEST INTO
   @C_PRODDEST_CDPRODUTO;

   WHILE (@@FETCH_STATUS = 0)
   BEGIN
       /* OPERACAO = ACRESCENTAR Nutrientes */
      IF (@P_OPERACAO = 'A') BEGIN
       /* Seleciona os nutrientes do produto origem */
       /* que o destino nao tenha                   */
         DECLARE C_NUTRIENTES CURSOR FOR
         SELECT
            O.CDPRODUTO,
            O.CDNUTRIENTE,
            O.QTNUTRPROD,
            O.DSFONTNUTR
         FROM
            NUTRPROD O, PRODUTO PR
         WHERE
            (PR.CDPRODUTO = O.CDPRODUTO)    AND
            (PR.CDARVPROD = @P_CDPRODORIG) AND
            (CHARINDEX(O.CDNUTRIENTE, @P_LISTANUTR) > 0) AND
            (NOT EXISTS (SELECT D.CDNUTRIENTE
                         FROM NUTRPROD D
                         WHERE
                         (D.CDPRODUTO   = @C_PRODDEST_CDPRODUTO ) AND
                         (D.CDNUTRIENTE = O.CDNUTRIENTE         )
                        )
            );
         OPEN C_NUTRIENTES;
         FETCH NEXT FROM C_NUTRIENTES INTO
            @C_NUTRIENTES_CDPRODUTO,
            @C_NUTRIENTES_CDNUTRIENTE,
            @C_NUTRIENTES_QTNUTRPROD,
            @C_NUTRIENTES_DSFONTNUTR;

         /* Acrescenta os novos nutrientes */
         WHILE (@@FETCH_STATUS = 0)
         BEGIN
            INSERT INTO NUTRPROD
            (CDPRODUTO, CDNUTRIENTE, QTNUTRPROD, DSFONTNUTR)
            VALUES
            (@C_PRODDEST_CDPRODUTO, @C_NUTRIENTES_CDNUTRIENTE, @C_NUTRIENTES_QTNUTRPROD, @C_NUTRIENTES_DSFONTNUTR);
            FETCH NEXT FROM C_NUTRIENTES INTO
               @C_NUTRIENTES_CDPRODUTO,
               @C_NUTRIENTES_CDNUTRIENTE,
               @C_NUTRIENTES_QTNUTRPROD,
               @C_NUTRIENTES_DSFONTNUTR;
         END;
         CLOSE C_NUTRIENTES;
         DEALLOCATE C_NUTRIENTES;
      END;

      /* OPERACAO = SUBSTITUIR Nutrientes */
      IF (@P_OPERACAO = 'S') BEGIN
      /* Deleta os nutrientes do produto */
         DELETE FROM NUTRPROD
         WHERE CDPRODUTO = @C_PRODDEST_CDPRODUTO;

      /* Seleciona os nutrientes do produto de origem */
         DECLARE C_NUTRIENTES CURSOR FOR
         SELECT
            NP.CDPRODUTO,
            NP.CDNUTRIENTE,
            NP.QTNUTRPROD,
            NP.DSFONTNUTR
         FROM
            NUTRPROD NP, PRODUTO PR
         WHERE
            (PR.CDPRODUTO = NP.CDPRODUTO) AND
            (PR.CDARVPROD = @P_CDPRODORIG) AND
            (CHARINDEX(CDNUTRIENTE, @P_LISTANUTR) > 0);
         OPEN C_NUTRIENTES;
         FETCH NEXT FROM C_NUTRIENTES INTO
               @C_NUTRIENTES_CDPRODUTO,
               @C_NUTRIENTES_CDNUTRIENTE,
               @C_NUTRIENTES_QTNUTRPROD,
               @C_NUTRIENTES_DSFONTNUTR;
         WHILE (@@FETCH_STATUS = 0)
         BEGIN

      /* Insere nutriente por nutriente no produto de destino */
            INSERT INTO NUTRPROD
            (CDPRODUTO, CDNUTRIENTE, QTNUTRPROD, DSFONTNUTR)
            VALUES
            (@C_PRODDEST_CDPRODUTO, @C_NUTRIENTES_CDNUTRIENTE, @C_NUTRIENTES_QTNUTRPROD, @C_NUTRIENTES_DSFONTNUTR);
            FETCH NEXT FROM C_NUTRIENTES INTO
                  @C_NUTRIENTES_CDPRODUTO,
                  @C_NUTRIENTES_CDNUTRIENTE,
                  @C_NUTRIENTES_QTNUTRPROD,
                  @C_NUTRIENTES_DSFONTNUTR;
         END;
         CLOSE C_NUTRIENTES;
         DEALLOCATE C_NUTRIENTES;
      END;
      FETCH NEXT FROM C_PRODDEST INTO
      @C_PRODDEST_CDPRODUTO;
   END;
   CLOSE C_PRODDEST;
   DEALLOCATE C_PRODDEST;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'EXPORTA_NUTRIENTES','4.11.065';
GO

